package mosaic.sim.contBlocks;

import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.continuous.*;
import mosaic.sim.kernel.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.JLabel;


/** Implements the absolute value. */
public class Abs extends AbstractContEquation{
   DoubleVariable input;DoubleVariable output;
   JLabel la;
   public Abs() {
      super();
      initPaint();
      } // Abs;

   public Abs(String name) {
      super(name);
      initPaint();
      } // Abs

   public Abs(DoubleVariable input,DoubleVariable output){
      super();
      initPaint();
      setInput(input);
      setOutput(output);
      } // Abs;

   public Abs(String name,DoubleVariable input,DoubleVariable output){
      super(name);
      initPaint();
      setInput(input);
      setOutput(output);
      } // Abs


  protected void initPaint(){
    la=new JLabel();
    la.setFont(new java.awt.Font("Dialog", 1, 10));
    la.setForeground(Color.black);
    la.setOpaque(false);
  }

  /** Set the input variable. */
  public void setInput(DoubleVariable input){
    try{
      this.input=input;
      setDomains(new DoubleVariable[]{input});
    } catch(VariableException e){printErrorOutput(e.getMessage());}
  }

  /** Set the output variable. */
  public void setOutput(DoubleVariable output){
    this.output=output;
    try{
      setTargets(new DoubleVariable[]{output});
    } catch(VariableException e){printErrorOutput(e.getMessage());}
  }

  /** Returns the input variable. */
  public DoubleVariable getInput(){
    return input;
  }

  /** Returns the output variable. */
  public DoubleVariable getOutput(){
    return output;
  }

  public void equation(){
    output.setValue(Math.abs(input.getValue()));
  }

  public void paintComponent(Graphics g) {
    Graphics2D g2=(Graphics2D) g;
    int h = getSize().height-1;
    int w = getSize().width-1;
    int minWH=Math.min(h,w/2);

    Color lineC;
    Color fillC;
/*    if (enabled) g.setColor(Color.black);
    else g.setColor(Color.gray);*/
    if (enabled){
      lineC=Color.black;
      fillC=Color.yellow;
    }
    else{
      lineC=Color.gray;
      fillC=Color.lightGray;
    }
    g2.setPaint(fillC);

//    Rectangle r=new Rectangle(w/4,0,w/2,h);
    Ellipse2D e=new Ellipse2D.Double(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    g2.fill(e);

    g2.setPaint(lineC);
    g2.draw(e);
    g2.draw(new Line2D.Double(0,h/2,w/2-minWH/2,h/2));
    g2.draw(new Line2D.Double(w/2+minWH/2,h/2,w,h/2));

//    la.setBounds((x1+x2)/2+5,(y1+y2)/2-10,(int)la.getPreferredSize().getWidth(),(int)la.getPreferredSize().getHeight());
    la.setHorizontalAlignment(la.CENTER);
    la.setVerticalAlignment(la.CENTER);

    la.setBounds(e.getBounds());
    Graphics2D g3=(Graphics2D)g2.create(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    la.setText("ABS");
    la.paint(g3);

    if (input!=null){
      la.setBounds(0,0,w/2-minWH/2,h/2);
      la.setText(input.getLabel());
      la.setFont(new java.awt.Font("Dialog", 0, 10));
      la.setVerticalAlignment(la.BOTTOM);
      la.setHorizontalAlignment(la.LEFT);
      g3=(Graphics2D)g2.create(0,0,w/2-minWH/2,h/2);
      la.paint(g3);
    }
    if (output!=null){
      la.setBounds(w/2+minWH/2,0,w/2-minWH/2,h/2);
      la.setText(output.getLabel());
      la.setHorizontalAlignment(la.RIGHT);
      g3=(Graphics2D)g2.create(w/2+minWH/2,0,w/2-minWH/2,h/2);
      if (input==null){
        la.setFont(new java.awt.Font("Dialog", 0, 10));
        la.setVerticalAlignment(la.BOTTOM);
      }
      la.paint(g3);
    }

  }

  public Dimension getPreferredSize() {
    return new Dimension(58,24);
  }


}
