package mosaic.sim.contBlocks;

import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.continuous.*;
import mosaic.sim.kernel.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.JLabel;
import java.util.ArrayList;


/** Implements the division. */
public class Division extends AbstractContEquation{
   DoubleVariable divident;DoubleVariable divisor;DoubleVariable output;
   JLabel la;
   public Division() {
      super();
      initPaint();
      } // Division;

   public Division(String name) {
      super(name);
      initPaint();
      } // Division


   public Division(DoubleVariable divident,DoubleVariable divisor,DoubleVariable output){
      super();
      initPaint();
      setDivident(divident);
      setDivisor(divisor);
      setOutput(output);
      } // Division;

   public Division(String name,DoubleVariable divident,DoubleVariable divisor,DoubleVariable output){
      super(name);
      initPaint();
      setDivident(divident);
      setDivisor(divisor);
      setOutput(output);
      } // Division

  protected void initPaint(){
    la=new JLabel();
    la.setFont(new java.awt.Font("Dialog", 1, 10));
    la.setForeground(Color.black);
    la.setOpaque(false);
  }

  /** Set the divident. */
  public void setDivident(DoubleVariable divident){
    try{
      this.divident=divident;
      ArrayList l=new ArrayList(2);
      if (divisor!=null) l.add(divisor);
      if (divident!=null) l.add(divident);
      DoubleVariable domains[]=new DoubleVariable[l.size()];
      for (int i=0;i<l.size();i++)domains[i]=(DoubleVariable)l.get(i);
      setDomains(domains);
    } catch(VariableException e){printErrorOutput(e.getMessage());}
  }

  /** Set the divisor. */
  public void setDivisor(DoubleVariable divisor){
    try{
      this.divisor=divisor;
      ArrayList l=new ArrayList(2);
      if (divisor!=null) l.add(divisor);
      if (divident!=null) l.add(divident);
      DoubleVariable domains[]=new DoubleVariable[l.size()];
      for (int i=0;i<l.size();i++)domains[i]=(DoubleVariable)l.get(i);
      setDomains(domains);
    } catch(VariableException e){printErrorOutput(e.getMessage());}
  }


  /** Set the output variable. */
  public void setOutput(DoubleVariable output){
    this.output=output;
    try{
      setTargets(new DoubleVariable[]{output});
    } catch(VariableException e){printErrorOutput(e.getMessage());}
  }

  /** Returns the divident. */
  public DoubleVariable getDivident(){
    return divident;
  }

  /** Returns the divisor. */
  public DoubleVariable getDivisor(){
    return divisor;
  }

  /** Returns the output variable. */
  public DoubleVariable getOutput(){
    return output;
  }

  public void equation(){
    output.setValue(divident.getValue()/divisor.getValue());
  }

  public void paintComponent(Graphics g) {
    Graphics2D g2=(Graphics2D) g;
    int h = getSize().height-1;
    int w = getSize().width-1;
    int minWH=Math.min(h,w/2);
    int dist=Math.max(h/10,2);

    Color lineC;
    Color fillC;
/*    if (enabled) g.setColor(Color.black);
    else g.setColor(Color.gray);*/
    if (enabled){
      lineC=Color.black;
      fillC=Color.yellow;
    }
    else{
      lineC=Color.gray;
      fillC=Color.lightGray;
    }
    g2.setPaint(fillC);

//    Rectangle r=new Rectangle(w/4,0,w/2,h);
    Ellipse2D e=new Ellipse2D.Double(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    g2.fill(e);

    g2.setPaint(lineC);
    g2.draw(e);
    g2.draw(new Line2D.Double(0,h/2+dist,w/2-minWH/2,h/2+dist));
    g2.draw(new Line2D.Double(0,h/2-dist,w/2-minWH/2,h/2-dist));
    g2.draw(new Line2D.Double(w/2+minWH/2,h/2,w,h/2));

//    la.setBounds((x1+x2)/2+5,(y1+y2)/2-10,(int)la.getPreferredSize().getWidth(),(int)la.getPreferredSize().getHeight());
    la.setHorizontalAlignment(la.CENTER);
    la.setVerticalAlignment(la.CENTER);

    la.setBounds(e.getBounds());
    Graphics2D g3=(Graphics2D)g2.create(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    la.setText("DIV");
    la.paint(g3);

    if (divident!=null){
      la.setBounds(0,0,w/2-minWH/2,h/2-dist);
      la.setText(divident.getLabel());
      la.setFont(new java.awt.Font("Dialog", 0, 10));
      la.setVerticalAlignment(la.BOTTOM);
      la.setHorizontalAlignment(la.LEFT);
      g3=(Graphics2D)g2.create(0,0,w/2-minWH/2,h/2);
      la.paint(g3);
    }
    if (divisor!=null){
      la.setBounds(0,h/2+dist,w/2-minWH/2,h);
      la.setText(divisor.getLabel());
      g3=(Graphics2D)g2.create(0,h/2+dist,w/2-minWH/2,h);
      if (divident==null){
        la.setHorizontalAlignment(la.LEFT);
        la.setFont(new java.awt.Font("Dialog", 0, 10));
      }
      la.setVerticalAlignment(la.TOP);
      la.paint(g3);
    }


    if (output!=null){
      la.setBounds(w/2+minWH/2,0,w/2-minWH/2,h/2);
      la.setText(output.getLabel());
      la.setHorizontalAlignment(la.RIGHT);
      g3=(Graphics2D)g2.create(w/2+minWH/2,0,w/2-minWH/2,h/2);
      if (divident==null&&divisor==null){
        la.setFont(new java.awt.Font("Dialog", 0, 10));
      }
      la.setVerticalAlignment(la.BOTTOM);
      la.paint(g3);
    }

  }

  public Dimension getPreferredSize() {
    return new Dimension(58,24);
  }


}