/**
 * Project mosaic
 *
 * ModelComponentEvent.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

public class ModelComponentEvent extends java.util.EventObject {

    // String constants for describing the events
    public static final String MODEL_COMPONENT_ADDED           ="modelComponentAdded";
    public static final String MODEL_COMPONENT_REMOVED         ="modelComponentRemoved";
    public static final String MODEL_COMPONENT_ENABLED         ="modelComponentEnabled";
    public static final String MODEL_COMPONENT_DISABLED        ="modelComponentDisabled";
    public static final String MODEL_COMPONENT_GOT_CONTINUOUS  ="modelComponentGotContinuous";
    public static final String MODEL_COMPONENT_GOT_DISCRETE    ="modelComponentGotDiscrete";
    public static final String MODEL_COMPONENT_CHANGED         ="modelComponentChanged";


    protected ModelComponent model;
    protected ModelComponent subModel;
    protected String eventName;
    protected double eventTime;

// Constructors ------------------------------------------------------------------------------------
    /** Constructor with the event source as parameter */
    public ModelComponentEvent(ModelComponent source) {
  	    this (source, "modelComponentEvent");
    }

    /** Constructor with the event source and the event name as parameter */
    public ModelComponentEvent(ModelComponent source, String iEventName){
  	    super(source);
  	    setModelComponent(source);
  	    setEventName(iEventName);
    }

    /** Constructor with the event source, the event name, and the involved subcomponent as parameter */
    public ModelComponentEvent(ModelComponent source, String iEventName, ModelComponent subModel){
  	    super(source);
  	    setModelComponent(source);
        setSubModelComponent(subModel);
  	    setEventName(iEventName);
    }

// model component ---------------------------------------------------------------------------------

    /** Gets the model component */
    public ModelComponent getModelComponent(){
  	    return model;
    }

    /** Sets the model component */
    public void setModelComponent (ModelComponent model) {
  	    this.model = model;
    }

// sub model component -----------------------------------------------------------------------------

    /** Gets the sub model component */
    public ModelComponent getSubModelComponent(){
  	    return subModel;
    }

    /** Sets the sub model component */
    public void setSubModelComponent (ModelComponent subModel) {
  	    this.subModel = subModel;
    }

// sub model component -----------------------------------------------------------------------------

    /** Gets the event name  */
    public String getEventName(){
  	    return eventName;
    }

    /** Sets the event name  */
    public void setEventName (String eventName) {
  	    this.eventName = eventName;
    }

// sub model component -----------------------------------------------------------------------------

    /** Gets the event time  */
    public double getEventTime(){
  	    return eventTime;
    }

    /** Sets the event time  */
    public void setEventTime (double eventTime) {
  	    this.eventTime = eventTime;
    }

}






