/**
 * Project mosaic
 *
 * StaticProjectSettings.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

/**
* Class providing access to the global settings for item processing applications.
* The settings are mainly used in PropertyEditors.
* Will be depracated when composer is finished.
*/
public class StaticProjectSettings {

  private StaticProjectSettings() {
  }

// item types for ItemGenerator customization ------------------------------------------------------

  protected static String[] itemTypes = new String[]
      {"sim.itemproc.BasicItem",
       "sim.itemproc.ConveyorItem",
       "sim.palette.Palette"};

  public static String[] getItemTypes () {
      return itemTypes;
  }

  public static void setItemTypes (String[] newItemTypes) {
      itemTypes = newItemTypes;
  }

// types of ReceiverSelection strategy objects for customizing ReceiverDecisionPoints etc.  --------

  protected static String[] receiverSelections = new String[]
      {"sim.itemproc.control.RandReceiverSelection",
       "sim.itemproc.control.RandNeedsItemSelection",
       "sim.itemproc.control.PercentageReceiverSelection",
       "sim.itemproc.control.PriorityNeedsItemSelection",
       "sim.itemproc.control.LongestNeedsItemSelection",
       "sim.itemproc.control.LeastRecentlyReceivedSelection"};

  public static void setReceiverSelections(String[] newReceiverSelections) {
    receiverSelections = newReceiverSelections;
  }

  public static String[] getReceiverSelections() {
    return receiverSelections;
  }


// types of ProviderSelection strategy objects for customizing ProviderDecisionPoints etc.  --------

  protected static String[] providerSelections= new String[]
      {"sim.itemproc.control.RandProviderSelection",
       "sim.itemproc.control.RandHasItemSelection",
       "sim.itemproc.control.PercentageProviderSelection",
       "sim.itemproc.control.PriorityHasItemSelection",
       "sim.itemproc.control.LongestHasItemSelection",
       "sim.itemproc.control.LeastRecentlyProvidedSelection"};

  public static void setProviderSelections(String[] newProviderSelections) {
    providerSelections = newProviderSelections;
  }

  public static String[] getProviderSelections() {
    return providerSelections;
  }

// WorkPiece operations to define the operations of WorkPieces  ------------------------------------

  protected static String[] workPieceOperations = new String[]
      {"drehen",
       "bohren",
       "schleifen"};


  public static String[] getWorkPieceOperations () {
      return workPieceOperations;
  }

  public static void setWorkPieceOperations (String[] newItemTypes) {
      itemTypes = workPieceOperations;
  }

  //
  // Simulation and Simulationscheduler Settings ---------------------------------------------------
  //

  protected final static String[] TIME_UNITS = new String[] {
                                                        "millisec",
                                                        "sec", //default
                                                        "min",
                                                        "hour",
                                                        "day",
                                                        "week"
                                                        };

  public static String[] getTimeUnits() {
    return TIME_UNITS;
  }

  //
  // Scheduling Modes
  //

  protected final static String[] SCHEDULING_MODES = new String[] {
                                                            "REALTIME", //def
                                                            "REALTIME2",
                                                            "REALTIME_NOHPC",
                                                            "FAST_NO_ANIM",
                                                            "FAST_NO_ANIM_NOHPC",
                                                            "AGENT_TED",
                                                            };
  public static String[] getSchedulingModes() {
    return SCHEDULING_MODES;
  }

  //
  // Integrator settings ---------------------------------------------------------------------------
  //

  protected final static String[] INTEGRATION_METHODS = new String[] {
                                                                "Euler",
                                                                "Heun", //def
                                                                "Runge-Kutta"
                                                              };
  public static String[] getIntegrationMethods() {
    return INTEGRATION_METHODS;
  }

} //================================================================================================
