package mosaic.sim.kernel.continuous;

import java.lang.*;
import java.util.*;
import java.lang.reflect.*;

/**
* Diese Klasse implementiert die mathematische Integrationsmethode von Heun. Es
* ist ein eher simples Verfahren, doch schon etwas genauer als jenes von Euler.
* Es ist durchaus schnell und die Fehler halten sich schon sehr in Grenzen.
*/

public class Heun
   extends BasicIntegrationMethod  {

   /**
   * Erzeugt eine Instanz der Klasse.
   */
   public Heun() {
      super();
   }

   /*
   * Initialisiert die Klassenvariablen aufgrund der an die Integrationsmethode
   * bergebenen Daten.
   */
   public void initialize(ArrayList stateArrayList, ArrayList equationsArrayList) {
      super.initialize(stateArrayList, equationsArrayList);

      int[] sizeArray = {2, stateArrayList.size()};
      k = (double[][])Array.newInstance(Double.TYPE, sizeArray);
   }


   /**
   * Fhrt eine Integrationsschritt aus
   */
   public void integrationStep(double currTime, double stepWidth)
      throws ContSimulationException {

      // compute k1
      for(int i = 0; i < nrStates; i++) {
         oldStates[i] = states[i].getValue();
         k[0][i] = stepWidth * states[i].getDerivative().getValue();
         states[i].setValue(oldStates[i] + k[0][i]);
      }

      // compute all variables
      doEquations();

      // compute k2
      for(int i = 0;i < nrStates;i++) {
         k[1][i] = stepWidth*states[i].getDerivative().getValue();
         states[i].setValue(oldStates[i] + (k[0][i] + k[1][i]) / 2);
         states[i].setChangedTime(currTime + stepWidth);
      }

      // compute all variables
      doEquations();

   } // integrationStep

} // class Euler

   
