package mosaic.sim.kernel.continuous;

import java.lang.*;
import java.util.*;
import java.lang.reflect.*;
import mosaic.sim.kernel.*;
/**
*
*/

public class RungeKutta4
   extends BasicIntegrationMethod
   {

   public RungeKutta4()
      { super(); }

   /*
   * Initialisiert die Klassenvariablen aufgrund der an die Integrationsmethode
   * bergebenen Daten.
   */
   public void initialize(ArrayList stateArrayList, ArrayList equationsArrayList) {
      super.initialize(stateArrayList, equationsArrayList);

      int[] sizeArray = {4, stateArrayList.size()};
      k = (double[][])Array.newInstance(Double.TYPE, sizeArray);
   }


   /**
   * Fhrt eine Integrationsschritt aus
   */
   public void integrationStep(double currTime, double stepWidth)
      throws ContSimulationException {

      // compute k0 = h*f(q), q' = q +k0/2
      for(int i = 0; i < nrStates; i++) {
         oldStates[i] = states[i].getValue();
         k[0][i] = stepWidth * states[i].getDerivative().getValue();
         states[i].setValue(oldStates[i] + k[0][i]/2);
      }
      // compute all variables
      doEquations();

      // compute k1 = h*f(q'), q' = q +k1/2
      for(int i = 0;i < nrStates;i++) {
         k[1][i] = stepWidth*states[i].getDerivative().getValue();
         states[i].setValue(oldStates[i] + k[1][i]/2);
      }
      // compute all variables
      doEquations();

      // compute k2 = h*f(q') , , q' = q + k2
      for(int i = 0;i < nrStates;i++) {
         k[2][i] = stepWidth*states[i].getDerivative().getValue();
         states[i].setValue(oldStates[i] + k[2][i]);
         states[i].setChangedTime(currTime + stepWidth);
      }

      // compute all variables
      doEquations();

      // compute k3 = h*f(q') , q' = q + (k0 + 2*k1 +2*k2 + k3)/6
      for(int i = 0;i < nrStates;i++) {
         k[3][i] = stepWidth*states[i].getDerivative().getValue();
         states[i].setValue(oldStates[i] + (k[0][i] + 2*k[1][i] + 2*k[2][i] + k[3][i]) / 6 );
         states[i].setChangedTime(currTime + stepWidth);
      }

      // compute all variables
      doEquations();

   } // integrationStep


} // class RungeKutta4

// end of RungeKutta4.java