
package mosaic.sim.kernel.continuous;

import java.util.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.simevent.*;

/**
* Class implementing a guard for a DoubleVariable variable crossing a value.
* <p>
* Implements the interface StateEventComponent.
* <b>Events:</b><p>
* <dl>
* <dt><code>zeroCrossingEvent</code></dt>
* <dd></dd>
* </dl>
*/

public class ValueCrossingGuard extends VariableGuard
   {


   public ValueCrossingGuard()
   { super();
     value = 0.0;
   }

   public ValueCrossingGuard(DoubleVariable real, double crossingValue)
      { super(real);
        value = crossingValue;
      }

   public ValueCrossingGuard(ModelComponent owner, String iLabel, DoubleVariable real, double crossingValue)
      { super(owner, iLabel, real);
        value = crossingValue;
      }


// crossing value to guard -------------------------------------------------------------------------

   /** double value, if real crosses this, a StateEvent will be released */
   protected double value;

   /**
   * Gets the crossing value
   */
   public double getCrossingValue()
      { return value; }

   /**
   * Sets the crossing value
   */
   public void setCrossingValue(double crossingValue)
      { value = crossingValue; }

// event occured -----------------------------------------------------------------------------------

	 /**
   * Checks if an event occured.
   */
   public boolean eventOccured() {
      if (!enabled)
         return false;
      else if (real==null)
         return false;
      else if ((real.getPreviousValue() >= value)&&(real.getValue() < value))
         return true;
      else if ((real.getPreviousValue() <= value)&&(real.getValue() > value))
         return true;
      else
         return false;
      }

	 /**
   * Trigger an event
   */
	   public void triggerEvent() {
      StateEvent stateEvent = new StateEvent(real);

      for(int loop=0;loop<listeners.size();loop++)
         ((StateEventListener)listeners.get(loop)).processSimEvent(stateEvent);
	    }

	 /**
   * Add SimEventListener
   */
   public void addSimEventListener(SimEventListener simEventListener) {
      if (!listeners.contains(simEventListener))
         listeners.add(simEventListener);
      }

	 /**
   * Remove SimEventListener
   */
   public void removeSimEventListener(SimEventListener simEventListener) {
      if (listeners.contains(simEventListener))
         listeners.remove(simEventListener);
      }

   /**
   * Add StateEventListener
   */
   public void addStateEventListener(StateEventListener stateEventListener) {
     if (!listeners.contains(stateEventListener))
         listeners.add(stateEventListener);
   }

	 /**
   * Remove StateEventListener
   */
   public void removeStateEventListener(StateEventListener stateEventListener) {
      if (listeners.contains(stateEventListener))
         listeners.remove(stateEventListener);
      }

   protected int sign(double value) {
      if (value < 0.0)
         return(-1);
      else if (value > 0.0)
         return(+1);
      else
         return(0);
      } // sign

  /** Gives a textual description of the value of the variable as debug information. */
  protected String getDebugText() {
    return super.getDebugText() + " at value " + value; 
  }

} // class ValueCrossingGuard


