package mosaic.sim.kernel.control;


import mosaic.util.*;
import mosaic.sim.*;

public class FixedTimeStrategy extends SchedulingStrategy {

    public static boolean[] DEBUG = new boolean[] {
                                                false,
                                                false,
                                                false, // out thread info
                                                false //create LOG Files
                                                };
    private long sleepTimeBetweenSteps;

    FixedTimeStrategy(SimulationScheduler scheduler) {
        this.scheduler = scheduler;
        hpCounter = new HighPerformanceCounter();
    }

    //
    // User Properties
    //
    public long getSleepTimeBetweenSteps() {
        return sleepTimeBetweenSteps;
    }

    //
    // Simulation
    //
    void reset() {
    }

    void performScheduling() {
        if (DEBUG[2]) SimInformer.printAllThreads();
        Thread.currentThread().setPriority(Thread.NORM_PRIORITY);

        /* run until scheduler says stop */
        do {
            scheduler.updateCurrentAnimationRate();
            sleepTimeBetweenSteps = (int)(1000.0 / scheduler.getAnimationRate() * 1000);
            scheduler.analyzer.newStep();
            hpCounter.resetTime();
            /* trigger the getSimApplication().getSimulator() to perform one step */
            scheduler.curSimTime = scheduler.getSimApplication().getSimulator().getNextSimulationTime();
            scheduler.getSimApplication().getSimulator().step();
            scheduler.analyzer.addStepDuration(hpCounter.getTime());
            hpCounter.resetTime();
            try {
                //user sets  1000 ==> 1   sec sleep per step
                //user sets 10000 ==> 0.1 sec sleep per step
                Thread.currentThread().sleep(sleepTimeBetweenSteps);
            } catch (InterruptedException e) {
                //fall through
            } finally {
                scheduler.analyzer.addSleepDuration(hpCounter.getTime());
            }
        } while(!scheduler.stopOnNextOccasion);
    }

    private HighPerformanceCounter hpCounter;

    private String ft(double in) {
        return ("" + ((int) (in * 1000)));
    }
}


