package mosaic.sim.kernel.control;

import mosaic.util.*;

public class RealTimeStrategyNoHPC extends SchedulingStrategy {

    final static double TOO_SLOW_TOLERANCE = 0.5;       // simtime

    public static boolean[] DEBUG = new boolean[] {
                                                false,
                                                false,
                                                false,  // out thread info
                                                false,  // create LOG Files
                                                false    // trace animationrealtime
                                                };

    //
    // Data + Higheperformancecounter
    //

    private double targetFps;              // Frames per second between steps
    private double currentFps;      // Frames per second between steps
    private double frameDuration;
    private int nrOfAnimations;
    private double slidingAvgCounter;

    //
    // Construction
    //

    RealTimeStrategyNoHPC(SimulationScheduler scheduler) {
        this.scheduler = scheduler;
        targetFps = 35;     // Frames per second between steps
        reset();
    }

    //
    // User Properties
    //
    public double getTargetFps() {
        return targetFps;
    }

    public void setTargetFps(double targetFps) {
        if (targetFps > 0.0) {
            this.targetFps = targetFps;
        }
    }

    //
    // Simulation
    //

    void reset() {
        currentFps = 20;                     // Frames per second between steps
        nrOfAnimations = 0;
        slidingAvgCounter = 0;
    }

    void performScheduling() {
        if (DEBUG[2]) SimInformer.printAllThreads();
        Thread.currentThread().setPriority(Thread.NORM_PRIORITY);
        double t2 = scheduler.getSimApplication().getSimulator().getNextSimulationTime();
        double simRealTimeSecs;

        ///* set curSimTime to StartTime */

        /* run until scheduler says stop */
        do {
            scheduler.updateCurrentAnimationRate();
            simRealTimeSecs = scheduler.getSimRealTime();

            if (DEBUG[3]) LogPrinter.println(4, "- simRealTime = " + ft(simRealTimeSecs));
            /* animate until the event time is reached */
            animateUntilSimTime(t2);
            if (DEBUG[3]) LogPrinter.println(4, "- T2          = " + ft(t2));
            scheduler.isLate = ((simRealTimeSecs - t2) > TOO_SLOW_TOLERANCE);
            if  (scheduler.isLate){
//                System.out.println(" Sim slow dev:"
//                     + (simRealTimeSecs - t2));
            }
            scheduler.analyzer.newStep();
            /* trigger the getSimApplication().getSimulator() to perform one step */
            scheduler.getSimApplication().getSimulator().step();
            /* get next Time */
            t2 = scheduler.getSimApplication().getSimulator().getNextSimulationTime();
        } while(!scheduler.stopOnNextOccasion);
    }

    //
    // Animation
    //

    private void animateUntilSimTime(double stopSimTime) {
        long stopRealTime;
        double animationDuration;
        double sleepDuration;
        long animateStartTime;
        double nextAnimTime;
        double realFrameDuration;
        double curSimTime;

        stopRealTime = System.currentTimeMillis() +
                            scheduler.toRealTimeDuration(stopSimTime);

        if (DEBUG[4]) System.out.println("animationRT for " + (stopRealTime - System.currentTimeMillis()));
        /* animate until time simTime happens */
        if (DEBUG[3]) LogPrinter.println(3, "****** ANIM START *******");
        while (System.currentTimeMillis() < (stopRealTime)) {

            //
            // Immediately after STEP() sleep until animation schould start
            //
            animateStartTime = System.currentTimeMillis();
            // perform Animation
            curSimTime = scheduler.getSimRealTime();
            if (DEBUG[3]) LogPrinter.println(4, "Ax = " + ft(curSimTime));
            scheduler.animator.animate(curSimTime);
            sleepDuration = 10;
            if (sleepDuration <= 0) {
                sleepDuration = 1;
                if (DEBUG[3]) LogPrinter.print(3, " **** SLEEP  <= 0 ****");
            }
            try {
                Thread.currentThread().sleep(Math.round(sleepDuration));
            } catch (InterruptedException e) {
            }

            nrOfAnimations++;

            // add data concerning this frame to the Analyzer
            scheduler.analyzer.newFrame();
        } //while
        if (DEBUG[3]) LogPrinter.println(3, "****** ANIM STOP *******");
        long postSleep = 10;
        try {
            Thread.sleep(postSleep);
            if (DEBUG[3]) LogPrinter.println(4, "POST = " + postSleep);
        } catch (InterruptedException e){
        }
    }

    private String ft(double in) {
        return ("" + ((int) (in * 1000)));
    }
}


