//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class VariableInputView    ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.input;

import mosaic.sim.kernel.output.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.control.*;
import mosaic.sim.kernel.simevent.*;
import java.util.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.*;
import javax.swing.*;

/**
* Base class to handle user input and forward it to userInputListeners
* Extends VariableView as a base implementation for the output
* view to visualize variable values.
*
*/

public class VariableInputView extends VariableView
implements UserEventComponent {

   /** contains all UserEventListeners */
   protected ArrayList listeners = new ArrayList();

	public VariableInputView() {
    super();
	}

  // tL ----------------------------------------------------------------------------------------------

   /**
   * Gets the time of last event
   */
   public double getTL()
      { return 0.0; }

   /**
   * Sets the time of last event
   */
   public void setTL(double newTime)
      { ; }


// select order  ---------------------------------------------------------------------------------------------

  /** variable to hold the selectOrder for the timer which is used as a priority
  * for tie breaking between several timer with equal next event time.
  */
  protected int selectOrder = 0;

  /** Gets the select order of the timer */
  public int getSelectOrder() {
      return selectOrder;
  }

  /** Sets the select order of the timer */
  public void setSelectOrder(int newOrder) {
    int oldSelectOrder = selectOrder;
    selectOrder= newOrder;
  }


// -----------------------------------------------------------------------------------------------------------
// interface userEventComponent --------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

   /**
   * Add SimEventListener
   */
   public void addSimEventListener(SimEventListener simEventListener) {
      if (!listeners.contains(simEventListener))
         listeners.add(simEventListener);
      }

	 /**
   * Remove SimEventListener
   */
   public void removeSimEventListener(SimEventListener simEventListener) {
      if (listeners.contains(simEventListener))
         listeners.remove(simEventListener);
      }

   /**
   * Add InputEventListener
   */
   public void addUserEventListener(UserEventListener userEventListener) {
     if (!listeners.contains(userEventListener))
         listeners.add(userEventListener);
   }

   public Object getUserInput()
   { return getValue();
   }

	 /**
   * Remove InputEventListener
   */
   public void removeUserEventListener(UserEventListener userEventListener) {
      if (listeners.contains(userEventListener))
         listeners.remove(userEventListener);
      }

// event -------------------------------------------------------------------------------------------

   /**
   * Trigger an event
   */
     public void triggerEvent() {
      UserEvent userEvent = new UserEvent(this);

      for(int loop=0;loop<listeners.size();loop++)
         ((SimEventListener)listeners.get(loop)).processSimEvent(userEvent);
	    }


// -----------------------------------------------------------------------------
// Trace  ----------------------------------------------------------------------
// -----------------------------------------------------------------------------

  /** variable to hold the model trace window */
/*  SimEventTraceWindow simEventTraceW;
  protected static int nrTraceWindows = 0;
*/

  /** inner class for opening TraceWindows which give out a trace of input changes */
/*  class TraceWindowOpener extends MouseAdapter {
        // variable to remember the number of trace windows created, used for positioning
        public void mouseClicked (MouseEvent e) {
            if (simEventTraceW == null) {
                Frame f = findContainingFrame();
                if (f != null & f instanceof Frame) {
                    simEventTraceW = new SimEventTraceWindow ((JFrame)f, getLabel(), VariableInputView.this);
                    addSimEventListener (simEventTraceW);
                    simEventTraceW.pack();
                    simEventTraceW.setModal(false);
                    simEventTraceW.setLocation(300 + 10*(nrTraceWindows%7) + 50*(nrTraceWindows/7),
                                            200 + 10*(nrTraceWindows%7));
                    nrTraceWindows = nrTraceWindows + 1;
                }
                else {
                    System.out.println(">> tried to open trace window without parent frame for model " + getLabel());
                }
            }
            if (simEventTraceW != null) {
                simEventTraceW.setVisible(true);
                simEventTraceW.toFront();
            }
        }
    }
*/

//trace: -----------------------------------------------------------------------------------------------------


  /** Puts out a line with state information in the trace window */
/*  public void trace ( ) {
    if (simEventTraceW != null && simEventTraceW.isShowing()) {
          simEventTraceW.printTrace("Time " + getTime() + ":  " + getLabel() + " event triggered \n");
      }

  }
*/


  /** Clears the trace window */
/*  public void clearTrace ( ) {
      if (simEventTraceW != null) {
          simEventTraceW.clearStateTrace();
      }
  }
*/


}
