
/**
* Animation view to visualize numeric values.
* Can serve as variable change listeners for numeric variables.
* Displays the number using drawString.
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/

package mosaic.sim.kernel.output;
import java.awt.*;
import java.util.*;
import java.beans.*;


public class FormattedNumberView extends NumberView{

    /** Default constructor NumberView */
    public FormattedNumberView() {
    }
    // paintValue ------------------------------------------------------------------------------------------------

    /**
    * Paints the number value;
    * by putting out the text representing the number in the view, formatted to
    * nrDigits after comma.
    * @param g     the graphics context of the view
    */
    public void paintValue(Graphics g) {
       int width = getSize().width;
       int height = getSize().height;
       g.setFont(getFont());
       g.setColor(getForeground());
       FontMetrics fm = g.getFontMetrics();
       // is number less than visible range (10 E-digits) ?
       String outText;
       if (( getNumber() == null) ||
           ( Math.abs(number.doubleValue()) < (Math.pow(10, 1 - digits))) ){
           outText = "0.0";
       } else {
           int toIndex = Math.min(getText().indexOf(".")+digits, getText().length());
           outText = (getText().substring(0, toIndex)) + unitString;
       }
       g.drawString( outText,
                   (width - fm.stringWidth(outText)) / 2,
                   (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
     }

     // Properties

     // digits -----------------------------------------------------------------
     int digits = 2;

     /**
     * Sets the new number of digits to be draw in view after comma.
     */
     public void setDigits(int d) {
       digits = d;
     }

     /**
     * Gets the actual minimum for values entered.
     */
     public int getDigits() {
       return digits;
     }

     // unitString  ------------------------------------------------------------
     String unitString = "";

     /**
     * Sets the new unitString.
     */
     public void setUnitString(String s) {
       unitString = s;
     }

     /**
     * Gets the actual unitString.
     */
     public String getUnitString() {
       return unitString;
     }

}
