
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class VariableView    ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.output;

import java.awt.*;
import java.util.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.variable.*;

/**
* Base class for the output view to visualize variable values.
* Extends BasicVariableObserver as a base implementation of VariableChangeListener. 
*
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G% 
*/
public class VariableView extends BasicVariableObserver {

  /** Default constructor ValueView */
  public VariableView() {
      setLabel (" ");
      setBackground(new Color(140, 200, 255));
      setForeground(Color.black); 
  }


// variableChange --------------------------------------------------------------------------------------------

  /**
  * Reacts to a variableChange event by setting the value and repainting the view.
  */
  public void variableChange(VariableChangeEvent e) {
      super.variableChange(e);
      repaint();
      }


// paint -----------------------------------------------------------------------------------------------------
 
  /**
  * Paints the view;
  * by calling <code>super.paint</code> which paints the background and
  * calling polymorphic method <code>paintValue</code> to delegate painting of the values to specialized views.
  *
  * @param g the graphics context of the view
  */
  public void paintComponent(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    g.setColor(getBackground());
    g.fill3DRect(0, 0, width, height, true);
    g.setColor(getForeground());
    g.setFont(getFont());
    FontMetrics fm = g.getFontMetrics();
    g.drawString(getLabel(), (width - fm.stringWidth(getLabel())) / 2, height - fm.getMaxDescent() -2);
    paintValue (g);
  }

  /** Empty method paintValue */
  public void paintValue(Graphics g) { }


// size ------------------------------------------------------------------------------------------------------

  /** Gets minimum size for the LayoutManager */
  public Dimension getMinimumSize() {
    return new Dimension(10,10);
  }

  /** Gets preferred size for the LayoutManager */
  public Dimension getPreferredSize() {
    return new Dimension(40,24);
  }


// double buffering ------------------------------------------------------------------------------------------

    /** Graphics object for off screen  painting */
    protected Graphics bufferG;
    /** image for off screen painting */
    Image offScreenBuffer;
}

