
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class ExponentialDistribution    --------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------
// programmed by: hp
// last modified by: hp, 29.12.98
// documented by: hp, 29.12.98

package mosaic.sim.kernel.random;

import java.lang.Math;

/**
* Class realizing an exponential distribtion.
* Has as parameter the mean value.
* <p> 
* <b>Customization </b>
* <ul>
* <li>Use setMean to set the mean value of the exponential distribution 
* </ul>
*/
public class ExponentialDistribution extends RandomDistribution
{
    /** Constructor intializing mean by 1.0. */
    public ExponentialDistribution () {
        setMean (1);
        setLabel("eD");
    }
    /** Constructor intializing mean by paramter iMean.  */
    public ExponentialDistribution (double iMean) {
        setMean (iMean);
        setLabel("eD");
    }
    
   /** Sets the mean of the distribution */
   public void setMean (double aMean) {
        mean = aMean;
    }
    
   /** Gets the mean of the distribution */
   public double getMean () {
        return mean;
    }
    
   /** Generates a random int from the exponential distribution  */
   public int getRandInt()  {
        return ((int) Math.round(-mean * (Math.log(randomSeed.nextDouble()))));
   }
    
   /** Generates a random double from the exponential distribution  */
   public double getRandDouble()  {
        return (-mean * Math.log(randomSeed.nextDouble()));
   }
    
   /** variables to hold mean  */
   protected double mean; 
}