
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class ConstantTimeSeed           --------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------
// programmed by: hp
// last modified by: hp, 29.12.98
// documented by: hp, 29.12.98

package mosaic.sim.kernel.random;

import mosaic.sim.kernel.control.*;
import mosaic.sim.kernel.*; 
/**
* Class realizing a Time seed which provides time intervals according to a list
* specified by the user.
*
* <b>Customization </b>
* <ul>
* <li>Use setTimeIntervals to set the sequence of time intervals to be generated.
* </ul>
**/
public class TimeSequence extends BasicRandomComponent
                                implements TimeSeed  {

  /** Constructor with initial array of time intervals */
  public TimeSequence (double [] newIntervals) {
      timeIntervals =  newIntervals;
  }

  /** Default constructor */
  public TimeSequence () {
  }

  /** Resets the TimeSequence by setting the cursor to the beginning of the sequence */
  public void reset () {
      super.reset();
      current = 0;
  }


  /** Gets next time span (to the next event by calling getRandDouble*/
  public double getTimeSpan() {
      if (current < timeIntervals.length) {
        double interval = timeIntervals[current];
        current++;
        return  interval;
      }
      else
        return ModelComponent.INF;
  }

  /** Sets the sequence of time intervals */
  public void setTimeIntervals(double[] newTimeIntervals) {
    timeIntervals = newTimeIntervals;
  }

  /** Gets the sequence of time intervals */
  public double[] getTimeIntervals( ) {
    return timeIntervals;
  }

  private double[] timeIntervals = new double[]{ };

  protected int current = 0;

}
