

//Project:        mosaic kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// Class SimEvent ------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------
// programmed by: hp
// last modified by: hp, 14.12.98
// documented by: hp, 8.4.99

package mosaic.sim.kernel.simevent;

import java.util.EventObject;


/**
* Event object for simulation events.
* SimEvents are meant realize the fundamental events in simulation models,
* like time scheduled events, state events in combined models and input events from a user in viusal interactive
* simulation.
* Carries the source and the time of the event.
* <p>
* Super class of TimeEvent, StateEvent, and InputEvent (FOR LATER USE).
*/
public class SimEvent extends EventObject {


// Constructors ----------------------------------------------------------------------------------------------

  /** Constructors for SimEvent initializing event source and time to 0. */
  public SimEvent (Object source) {
        super (source);
        setTime (0);
  }

  /** Constructors for SimEvent initializing event source and event time */
  public SimEvent (Object source, double evtTime) {
        super (source);
        setTime (evtTime);
  }


// time ------------------------------------------------------------------------------------------------------

  /** Gets the time of the event. */
  public double getTime () {
      return time;
  }
      
  /** Sets the time of the event. */
  public void setTime (double newTime) {
      time = newTime;
  }

  /** variable to hold the time of the event. */
  protected double time;

}   // end SimEvent -----------------------------------------------------------------------------------------
