
//Project:        mosaic kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// interface TimeEventComponent -------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------
// programmed by: hp
// last modified by: hp, 14.12.98
// documented by: hp, 14.12.98

package mosaic.sim.kernel.simevent;

import mosaic.sim.util.artificialidentifier.ArtificialIDOwner;

/**
* Interface which must be supported by all active, schedulable event firing simulation components.
* Extends SimEventComponent
* This interface is used by the Simulator to schedule and maintain in event list and trigger them.
* Implement this interface to realize any active, schedulable simulation components.
* Is implemented by class Timer and BasicActiveModel.
* <p>
* TimeEventComponents must provide:
* <ul>
* <li> tN (time of next event) to hold the time of the next scheduled event in this component.
* </ul>
* <p>
* @see sim.kernel.Timer
* @see sim.kernel.BasicActiveModel
*/
public interface TimeEventComponent extends SimEventComponent, ArtificialIDOwner {

  /** Gets the time of next event. */
  public double getTN();
    /** Sets the time of next event. */
  public void setTN(double newTime);


// TimeEvent -------------------------------------------------------------------------------------------------

  /** Add TimeEventListener */
  public void addTimeEventListener (mosaic.sim.kernel.simevent.TimeEventListener l);

  /** Remove TimeEventListener */
  public void removeTimeEventListener (mosaic.sim.kernel.simevent.TimeEventListener l);

// select order  -------------------------------------------------------------------------------------------------

  /** Sets select order in case of simultansous events */
  public void setSelectOrder (int order);

  /** Gets select order in case of simultansous events */
  public int getSelectOrder ();

}
