package mosaic.sim.kernel.statistics;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;
import mosaic.sim.kernel.control.*;

/**
* Class for not time weighted statistics beans of number values.
* Computes max, min, mean and std devition of the values received per observation.
* <p>
* Use this class for computing min, max, mean and std devition of the values of numeric properties
* per observation (is not time weighted, mean turnaroung time per observation).
* <p>
* <b>Bound properties:</b>
* <dl>
* <dt><code> min      </code></dt> <dd> the minimum value received </dd>
* <dt><code> max      </code></dt> <dd> the maximum value received </dd>
* <dt><code> mean     </code></dt> <dd> the mean of the values received </dd>
* <dt><code> stdDev   </code></dt> <dd> the std deviaion of  values received </dd>
* </dl>
*
* @see   TimeDependentStatistic
*
* @author Herbert Praehofer
* @version %I% %G%
*/
public class ObservationalStatistic extends BasicStatistic
{

  /** Default constructor StatisticsBean */
  public ObservationalStatistic() {
  }


// mean   ----------------------------------------------------------------------------------------------------

  /** variable to hold the mean value */
  double mean;

  /** Gets the mean value */
  public double getMean () {
  		return mean;
  }

  /** sets the mean value */
  protected void setMean (double newMean) {
  		mean = newMean;
        // FOR LATER USE: ---> add signaling property change here
  }



// stdDev ----------------------------------------------------------------------------------------------------

  /** variable to hold the stdDeviation value */
  double stdDev;

  /** Gets the stdDev value */
  public double getStandardDeviation () {
  		return stdDev;
  }

  /** Sets the stdDev value */
  protected void setStandardDeviation (double newStdDev) {
  		stdDev = newStdDev;
        // ---> add signaling property change here
  }


// sum    ----------------------------------------------------------------------------------------------------

  /** variable to hold the sum of the values (= values * time)  */
  double sum;

  /** Gets the sum value */
  public double getSum () {
     return sum;
  }

  /** Sets the sum value */
  protected void setSum (double newSum) {
     sum = newSum;
  }

// sumOfSquares  ---------------------------------------------------------------------------------------------

  /** variable to hold the sum of the values (= values * time)  */
  double sumOfSquares;

  /** Gets the sumOfSquares value */
  public double getSumOfSquares () {
     return sumOfSquares;
  }

  /** Sets the sumOfSquares value */
  protected void setSumOfSquares (double newSumOfSquares) {
     sumOfSquares = newSumOfSquares;
  }


// obs    ----------------------------------------------------------------------------------------------------

  /** variable to hold the time of last value change  */
  int obs;

  /** Gets the obs value */
  protected int getObs () {
      return obs;
  }

  /** Sets the obs value */
  protected void setObs (int newObs) {
        obs = newObs;
  }
  /** Increments the obs value */
  protected void incObs () {
     setObs (obs + 1);
  }


// computeStatistics -----------------------------------------------------------------------------------------


  /** Computes the statistics based on the received number values */
  protected void computeStatistics() {
    double val = number.doubleValue();
    super.computeStatistics();
    incObs();
    setSum (getSum() + val);
    setSumOfSquares (getSumOfSquares() + val*val);

    if (getObs() != 0)
        setMean(getSum() / getObs());

    if (getObs() > 1)
        setStandardDeviation (Math.sqrt(Math.abs((getSumOfSquares() - mean*mean*obs) / (getObs()-1))));
    else
        setStandardDeviation (0.0);

    setTL (getTime());
  }


// reset -----------------------------------------------------------------------------------------------------

  /** Clears the statistics */
  public void clearStatistics() {
    super.clearStatistics();
    setSum(0);
    setSumOfSquares(0);
    setMean (0);
    setStandardDeviation (0);
    setObs (0);
  }


}

