/*
 *  BooleanVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.lang.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.simevent.*;

/**
* BooleanVariable is a Variable for values of type boolean.
* Inherits the features of Variable, in particular: to signal variableChangeEvents.
* Has a boolean value and <code>getValue</code> and <code>setValue</code> to read and write it. 
* Use this bean class to realize simulation model variables of type boolean.
*
* @see sim.kernel.Variable
*/
public class BooleanVariable
   extends mosaic.sim.kernel.variable.AbstractVariable
{

// Constructors ------------------------------------------------------------------------------------

  /** Default constructor for BooleanVariable*/
   public BooleanVariable() {
        this("", VARIABLE, "", "", "", false);
      }

  /** Constructor for BooleanVariable initializing model owner and value. */
   public BooleanVariable (boolean value) {
        this("", VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, and value. */
   public BooleanVariable (String name, boolean value) {
        this(name, VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, variable type, and value. */
   public BooleanVariable (String name, int type, boolean value) {
        this(name, type, "", "", "", value);
     }

  /** Constructor for BooleanVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type, and value. */
  public BooleanVariable(String name, int type,
                    String unit, String quantity, String displayUnit, boolean value ) {
      super(name, type, unit, quantity, displayUnit);
      this.value = value;
   }

// boolean value      ------------------------------------------------------------------------------

  /** field to hold the value of the variable */
  protected boolean value = false;

  /** Gets the boolean value of the variable */
  public final boolean getValue () {
    return ((BooleanVariable)definedBy).value;
  }

  /** Sets the boolean value of the variable */
  public final void setValue (boolean newValue) {
    if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
      getSimApplication().getIntegrator().stateEventOccurred = true;
      return;
    }
    value = newValue;
    if (definedBy==this) signalVariableChange();
  }


// getValueObject      -----------------------------------------------------------------------------

  /** gets the value of the variable as type Object */
  public final Object getValueObject () {
      return new Boolean(getValue());
  }

// overwrite method processSimEvent from interface InputEventListener  -----------------------------

  /** Reacts to SimEvents from SimEventComponents.
   *  The events should be triggered by a inputEventSource and
   *  return Boolean-Objects.
   */
  public void processSimEvent (SimEvent evt){
    Object obj;
    if ((evt.getSource()) instanceof InputEventComponent){
      obj = ((InputEventComponent)(evt.getSource())).getInput();
      if (obj instanceof Boolean)
        this.setValue( ((Boolean)(obj)).booleanValue() );
    }
  }

}

