/*
 *  StringVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */

package mosaic.sim.kernel.variable;

import java.lang.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.simevent.*;


/**
* StringVariable is a Variable for values of type String.
* Use this bean class to realize simulation model variables of type String.
* Has a String value field and <code>getValue</code> and <code>setValue</code> methods to read and write it. 
* Inherits the features of Variable, in particular: to signal variableChangeEvents.
*
* @see sim.kernel.Variable
*/
public class StringVariable
   extends mosaic.sim.kernel.variable.AbstractVariable
{

// Constructors ------------------------------------------------------------------------------------


  /** Default constructor for BooleanVariable*/
   public StringVariable() {
        this("", VARIABLE, "", "", "", "");
      }

  /** Constructor for BooleanVariable initializing model owner and value. */
   public StringVariable (String value) {
        this("", VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, and value. */
   public StringVariable (String name, String value) {
        this(name, VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, variable type, and value. */
   public StringVariable (String name, int type, String value) {
        this(name, type, "", "", "", value);
     }

  /** Constructor for BooleanVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type, and value. */
  public StringVariable(String name, int type,
                    String unit, String quantity, String displayUnit, String value ) {
      super(name, type, unit, quantity, displayUnit);
      this.value = value;
   }


// String value      -------------------------------------------------------------------------------

  /** field to hold the value of the variable */
    protected String value = " ";

    /** Gets the String value of the variable */
    public final String getValue () {
      return ((StringVariable)definedBy).value;
    }

    /** Sets the String value of the variable */
    public final void setValue (String newValue) {
      if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
        getSimApplication().getIntegrator().stateEventOccurred = true;
        return;
      }
      value = newValue;
      if (definedBy==this)signalVariableChange();
    }


// getValueObject      -----------------------------------------------------------------------------

  /** gets the value of the variable as type Object */
    public final Object getValueObject () {
      return ((StringVariable)definedBy).value;
    }

    // overwrite method processSimEvent from interface InputEventListener  -------------------------

  /** Reacts to SimEvents from SimEventComponents.
   *  The events should be triggered by a inputEventComponent and
   *  return String-Objects.
   */
  public void processSimEvent (SimEvent evt){
    Object obj;
    if ((evt.getSource()) instanceof InputEventComponent){
      obj = ((InputEventComponent)(evt.getSource())).getInput();
      if (obj instanceof String)
        this.setValue( (String)obj );
    }
  }

}

