/**
 * Project mosaic
 *
 * VariableChangeComponent.java
 *
 * copyright 2000, JKU Linz, Austria
 *
 */

package mosaic.sim.kernel.variable;

import mosaic.sim.kernel.ModelComponent;
/**
* Interface which must be supported by all components firing variable change events.
* Extended by Variable.
* <p>
* Specifies methods to add and remove VariableChangeListener,
* a method getValueObject to get the current value as Object type, and
* a method to get the simulation time of the last value change.
*
* @see <{mosaic.sim.kernel.variable.Variable}>
*/
public interface VariableChangeComponent extends ModelComponent {

   /** Add a VariableChangeListener  */
   public void addVariableChangeListener (mosaic.sim.kernel.variable.VariableChangeListener l);

   /** Remove a VariableChangeListener */
   public void removeVariableChangeListener (mosaic.sim.kernel.variable.VariableChangeListener l);

  /** Gets the time of the last change in the variable */
  public double getChangedTime ();

  /** gets the value of the variable as type Object */
  public Object getValueObject ();

} // end VariableChangeCompoennt 

