/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: NeuronInput.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.2 $
 *  Last changed at: $Date: 2001/09/28 10:23:55 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron;

/** This Interface specifies the capability of an object to act like an
 *  input of a neuron, i.e. the ability to receive spike events.
 *  It will usually be directly connected to an object with the capability
 *  of sending spike events, i.e. an object that implements the NeuronOutput
 *  interface.
 *  @see NeuronOutput
 */
public interface NeuronInput {
	/** Receive a simple spike event. This function will be called by the
	 *  sender of the spike (an object that implements the NeuronOutput
	 *  interface) whenever it emits a spike event.
	 */
	public void spikeEvent(NeuronOutput sender);
}
