/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: NeuronOutput.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.3 $
 *  Last changed at: $Date: 2001/09/28 10:23:55 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron;

/** This interface specifies the capability of an object to act like the output
 *  part of a neuron, i.e. the ability to send spike events.
 *  Objects implementing the NeuronInput interface will register to this event
 *  source and all objects implementing NeuronOutput must call the spikeEvent()
 *  method of all currently registered objects whenever the object emits a
 *  spike event.
 *  @see NeuronInput
 */
public interface NeuronOutput {
	/** Add a spike event listener.
	 */
	public void addOutput(NeuronInput receiver);

	/** Remove a spike event listener.
	 */
	public boolean removeOutput(NeuronInput receiver);
}
