/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: Threshold.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.6 $
 *  Last changed at: $Date: 2001/10/03 07:25:12 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron;

import mosaic.sim.kernel.*;

/** This class represents a Neuron threshold. It is abstract and does not
 *  implement any behaviour, but only defines the interface that the Neuron
 *  object uses to interact with its associated Threshold object.
 */
public abstract class Threshold extends ActiveVariable {
    /** Creates a Threshold object. */
	public Threshold () {
		super();
	}

	/** Creates a Threshold object with a name (mainly useful during
	 *  debugging).
	 */
	public Threshold(String name) {
		super(name);
	}

	/** Returns the next fire time of the associated Neuron, that is the first
	 *  intersection between the threshold function and the given potential
	 *  function.
	 *  @param potential The potential function to compute the intersection
	 *                   with.
	 */
	public abstract double calculateNextFireTime(Potential potential);

	/** This method can be overridden to process the firing event, but super()
	 *  should be called */
	public void fired() {
		signalVariableChange();
	}

	/** Compute the current Neuron threshold from the gradient changes.
	 */
	public abstract double getCurrentThreshold();

	/** This implementation of AbstractVariable.getValueObject just returns the
	 *  current object.
	 */
	public Object getValueObject() {
		return this;
	}
}
