/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: PLNeuronInternalViewTest1.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.6 $
 *  Last changed at: $Date: 2002/02/20 00:35:55 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron.examples;

import mosaic.sim.*;
import mosaic.sim.kernel.*;
import mosaic.sim.neuron.*;
import mosaic.sim.neuron.netinputs.*;
import mosaic.sim.neuron.visualisation.*;
import java.awt.*;
import javax.swing.*;

public class PLNeuronInternalViewTest1 extends SimApplication {

	public PLNeuronInternalViewTest1() {
		SimpleMainFrame frame = new SimpleMainFrame(this);

// ========> create your panel here <==========================================//
		PLNeuronInternalViewTest1Panel p=new PLNeuronInternalViewTest1Panel();
		frame.setModelPanel(p);
// ============================================================================//

		frame.pack();
		frame.setVisible(true);
		frame.setTitle("Neuron internal view test 1");
		reset();
	}

	public static void main(String[] args) {
		PLNeuronInternalViewTest1 plNeuronInternalViewTest11 = new PLNeuronInternalViewTest1();
	}

}

class PLNeuronInternalViewTest1Panel extends ModelPanel {
	RandomSpikeSource s1 = new RandomSpikeSource(10);
	RandomSpikeSource s2 = new RandomSpikeSource(15);
	Neuron n1 = new Neuron();
	Neuron n2 = new Neuron();
	ViewSynchronizer sync = new ViewSynchronizer();
	ScrollingView[] views = new ScrollingView[6];

	public PLNeuronInternalViewTest1Panel() {
		try {
			jbInit();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

	private void jbInit() throws Exception {
		//ConstantPLSynapse master = new ConstantPLSynapse(null, null, 1, 0);
		//n1.setMasterSynapse(master);
		n1.setLabel("n1");
		n1.setName("n1");
		//n1.setLayout(null);
		//n2.setMasterSynapse(master);
		n2.setLabel("n2");
		n2.setName("n2");
		//n2.setLayout(null);

		PLPotential p1 = new PLPotential(50);
		p1.setName("p1");
		n1.setPotential(p1);
		PLPotential p2 = new PLPotential(50);
		p2.setName("p2");
		n2.setPotential(p2);

		ConstantPLThreshold t1 = new ConstantPLThreshold();
		ConstantPLThreshold t2 = new ConstantPLThreshold();
		t1.setName("t1");
		n1.setThreshold(t1);
		t2.setName("t2");
		n2.setThreshold(t2);
		ConstantPLThreshold.setAbsoluteRefractionTime((float) 0.5);
		ConstantPLThreshold.setQuiescentValue(5);
		ConstantPLThreshold.setRelativeRefractionStartValue(5);
		ConstantPLThreshold.setRelativeRefraction(new PLFunction(0));
		ConstantPLThreshold.calculateRelativeRefractionTime();

		s1.setLabel("s1");
		s1.setName("s1");
		//s1.setLayout(null);
		s2.setLabel("s2");
		s2.setName("s2");
		//s2.setLayout(null);

		s1.init();
		s2.init();

		ConstantPLSynapse sy1 = new ConstantPLSynapse(s1, n1, 2, 0);
		sy1.setLabel("sy1");
		ConstantPLSynapse sy2 = new ConstantPLSynapse(s1, n2, 1, 0);
		sy2.setLabel("sy2");
		ConstantPLSynapse sy3 = new ConstantPLSynapse(s2, n1, 1, 0);
		sy3.setLabel("sy3");
		ConstantPLSynapse sy4 = new ConstantPLSynapse(s2, n2, 2, 0);
		sy4.setLabel("sy4");

		PLFunction synapseResponse = new PLFunction(3);
		synapseResponse.gradients[0] = 1.5;
		synapseResponse.gradientTimes[0] = 0;
		synapseResponse.gradients[1] = -1;
		synapseResponse.gradientTimes[1] = 1;
		synapseResponse.gradients[2] = 0;
		synapseResponse.gradientTimes[2] = synapseResponse.getEndTime(0, 2);
		System.out.println("Synapse response function set to:");
		System.out.print(synapseResponse.toString());
		ConstantPLSynapse.setResponse(synapseResponse);

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(s1);
		this.add(s2);
		this.add(n1);
		this.add(n2);

		views[0] = new SpikeView("s1");
		views[1] = new SpikeView("s2");
		views[2] = new SpikeView("s3");
		views[3] = new SpikeView("s4");
		views[4] = new PLNeuronInternalView("v5", true, true);
		views[5] = new PLNeuronInternalView("v6", true, true);
		for (int i=0; i<6; i++) {
			views[i].setName("v" + (i+1));
			views[i].setTMax(50);
			sync.addView(views[i]);
			this.add(views[i]);
			if (views[i] instanceof PLNeuronInternalView) {
				views[i].setYMin(-1);
				views[i].setYMax(15);
			}
		}

		s1.addOutput((SpikeView) views[0]);
		s2.addOutput((SpikeView) views[1]);
		n1.addOutput((SpikeView) views[2]);
		n2.addOutput((SpikeView) views[3]);
		n1.getPotential().addVariableChangeListener((PLNeuronInternalView) views[4]);
		n1.getThreshold().addVariableChangeListener((PLNeuronInternalView) views[4]);
		n2.getPotential().addVariableChangeListener((PLNeuronInternalView) views[5]);
		n2.getThreshold().addVariableChangeListener((PLNeuronInternalView) views[5]);
	}
}
