/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: TemporalCodingTest1.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.8 $
 *  Last changed at: $Date: 2002/04/18 14:39:03 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron.examples;

import mosaic.sim.*;
import mosaic.sim.kernel.*;
import mosaic.sim.neuron.*;
import mosaic.sim.neuron.netinputs.*;
import mosaic.sim.neuron.netoutputs.*;
import java.awt.*;
import java.util.*;

public class TemporalCodingTest1 extends SimApplication {

	public TemporalCodingTest1() {
		SimpleMainFrame frame = new SimpleMainFrame(this);

// ========> create your panel here <==========================================//
		TemporalCodingTest1Panel p=new TemporalCodingTest1Panel();
		frame.setModelPanel(p);
// ============================================================================//

		frame.pack();
		frame.setVisible(true);
		reset();
	}

	public static void main(String[] args) {
		TemporalCodingTest1 test1 = new TemporalCodingTest1();
	}
}

class TemporalCodingTest1Panel extends ModelPanel {
	private final static int vectorLength = 5;

	private TemporalCodedInput inWithRef;
	private TemporalCodedInput inWithoutRef;
	private TemporalCodedOutput outWithRef;
	private TemporalCodedOutput outWithoutRef;

	private double[] lastVectorWithRef = new double[vectorLength];
	private double[] lastVectorWithoutRef = new double[vectorLength];
	private Random rand = new Random();

	public TemporalCodingTest1Panel() {
		try {
			jbInit();

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

	private void jbInit() throws Exception {
		inWithRef = new TemporalCodedInput(vectorLength, 1, true);
		inWithoutRef = new TemporalCodedInput(vectorLength, 1, false);
		outWithRef = new TemporalCodedOutput(vectorLength, 1, true, 10);
		outWithoutRef = new TemporalCodedOutput(vectorLength, 1, false, 10);
		add(inWithRef);
		add(inWithoutRef);
		add(outWithRef);
		add(outWithoutRef);

		// directly connect inputs and outputs
		inWithRef.getReferenceOutput().addOutput(outWithRef.getReferenceInput());
		for (int i=0; i<vectorLength; i++) {
			inWithRef.getOutput(i).addOutput(outWithRef.getInput(i));
			inWithoutRef.getOutput(i).addOutput(outWithoutRef.getInput(i));
		}

		for (int i=0; i<vectorLength; i++) {
			lastVectorWithRef[i] = rand.nextDouble();
			lastVectorWithoutRef[i] = rand.nextDouble();
		}

		outWithRef.addVectorOutputListener(
			new VectorOutputListener() {
				public void outputEvent(double[] vector) {
					Debug.assertCond(lastVectorWithRef.length == vector.length,
						"TemporalCodingTest1Panel.jbInit.OutputEventListener1",
						"array lengths to not match");
					for (int i=0; i<vector.length; i++)
						Debug.assertCond(Math.abs(lastVectorWithRef[i] - vector[i]) <=
						GlobalConstants.DOUBLE_INACCURACY_WINDOW,
						"TemporalCodingTest1Panel.jbInit.OutputEventListener1",
						"vector does not match at index " + i + ": " +
						vector[i] + " received, should be " +
						lastVectorWithRef[i]);

					for (int i=0; i<lastVectorWithRef.length; i++) {
						lastVectorWithRef[i] = rand.nextDouble();
					}

					try {
						inWithRef.enterVector(lastVectorWithRef);
					}
					catch (Exception e) {
						  e.printStackTrace();
						  Debug.assertCond(false,
							"TemporalCodingTest1Panel.init",
							"caught exception");
					}
				}
			} );

		outWithoutRef.addVectorOutputListener(
			new VectorOutputListener() {
				public void outputEvent(double[] vector) {
					Debug.assertCond(lastVectorWithoutRef.length == vector.length,
						"TemporalCodingTest1Panel.jbInit.OutputEventListener2",
						"array lengths to not match");
					/* This can in principle not work because of the missing
					   scaling - think about it. */
					/*for (int i=0; i<vector.length; i++)
						Debug.assert(Math.abs(lastVectorWithoutRef[i] - vector[i]) <=
						GlobalConstants.DOUBLE_INACCURACY_WINDOW,
						"TemporalCodingTest1Panel.jbInit.OutputEventListener2",
						"vector does not match at index " + i + ": " +
						vector[i] + " received, should be " +
						lastVectorWithoutRef[i]);*/

					for (int i=0; i<lastVectorWithoutRef.length; i++) {
						lastVectorWithoutRef[i] = rand.nextDouble();
					}

					try {
						inWithoutRef.enterVector(lastVectorWithoutRef);
					}
					catch (Exception e) {
						  e.printStackTrace();
						  Debug.assertCond(false,
							"TemporalCodingTest1Panel.init",
							"caught exception");
					}
				}
			} );
	}

	public void reset() {
		super.reset();
		init();
	}

	public void init() {
		try {
			inWithRef.enterVector(lastVectorWithRef);
			inWithoutRef.enterVector(lastVectorWithoutRef);
		}
		catch (Exception e) {
			e.printStackTrace();
			Debug.assertCond(false,
				"TemporalCodingTest1Panel.init",
				"caught exception");
		}
	}

	/** This is for sending out the next input event */
	/*public void processSimEvent(SimEvent e) {
		init();
	}*/
}
