/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: ImageOutput.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.4 $
 *  Last changed at: $Date: 2002/05/19 17:20:55 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron.netoutputs;

import java.awt.image.*;
import javax.media.jai.*;
import mosaic.sim.neuron.*;

/** This class receives vector or matrix output events from some spike decoder
 *  and transforms the values into an image, which is immediately saved in the
 *  file given to the constructor.
 *
 *  This class needs the JAI (Java Advanced Imaging) library to do its work.
 *  @see VectorOutputListener
 *  @see MatrixOutputListener
 *  @see mosaic.sim.neuron.netinputs.ImageInput
 */
public class ImageOutput
	   implements VectorOutputListener, MatrixOutputListener {
	protected ParameterBlockJAI savePB;
	protected int width;
	protected ColorModel colorModel;
	protected SampleModel sampleModel;

	public ImageOutput(String filename, String format, int width,
		   ColorModel colorModel, SampleModel sampleModel) {
		savePB = new ParameterBlockJAI("filestore");
		savePB.set(filename, "filename");
		savePB.set(format, "format");

		this.width = width;
		this.colorModel = colorModel;
		this.sampleModel = sampleModel;
	}

	/** Implementation of VectorOutputListener.outputEvent. */
	public void outputEvent(double[] vector) {
		Debug.assertCond(vector.length % width == 0,
			"ImageOutput.outputEvent",
			"Image must be rectangular. Given width does not match.");

		TiledImage image = new TiledImage(0, 0, width, vector.length / width,
				   0, 0, sampleModel, colorModel);

		for (int i=0; i<vector.length; i++) {
			image.setSample(i % width, i / width, 0, (int) vector[i]);
		}
		savePB.setSource(image, 0);
		JAI.create("filestore", savePB);
	}

	/** Implementation of MatrixOutputListener.outputEvent. */
	public void outputEvent(double[][] matrix) {
		TiledImage image = new TiledImage(0, 0, matrix.length, matrix[0].length,
				   0, 0, sampleModel, colorModel);

		for (int i=0; i<matrix.length; i++) {
			for (int j=0; j<matrix[0].length; j++) {
				image.setSample(j, i, 0, (int) matrix[i][j]);
			}
		}
		savePB.setSource(image, 0);
		JAI.create("filestore", savePB);
	}
}
