/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: MultiCell.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.5 $
 *  Last changed at: $Date: 2002/02/20 00:36:31 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron.simulations.genesis;

import mosaic.sim.*;
import mosaic.sim.kernel.*;
import mosaic.sim.neuron.*;
import mosaic.sim.neuron.netinputs.*;
import mosaic.sim.neuron.visualisation.*;
import java.awt.*;
import javax.swing.*;

/* This simulation tries to imitate the "MultiCell" example simulation
 * distributed with the GENESIS simulation toolkit as closely as possible.
 * It is a simple example that shall just show that this simulation framework
 * can produce qualitatively comparable results to GENESIS simulations.
 *
 * ======= Description of the simulation from the GENESIS simulator =======
 *
 */

public class MultiCell extends SimApplication {

	public MultiCell() {
		SimpleMainFrame frame = new SimpleMainFrame(this);

// ========> create your panel here <==========================================//
		MultiCellPanel p=new MultiCellPanel();
		frame.setModelPanel(p);
// ============================================================================//

		frame.pack();
		frame.setVisible(true);
		frame.setTitle("GENESIS MultiCell simulation");
		reset();
	}

	public static void main(String[] args) {
		MultiCell app = new MultiCell();
	}

}

class MultiCellPanel extends ModelPanel {
	Neuron n1 = new Neuron();
	Neuron n2 = new Neuron();
	ViewSynchronizer sync = new ViewSynchronizer();
	ScrollingView[] views = new ScrollingView[4];

	public MultiCellPanel() {
		try {
			jbInit();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

	private void jbInit() throws Exception {
		//ConstantPLSynapse master = new ConstantPLSynapse(null, null, 1, 0);
		//n1.setMasterSynapse(master);
		n1.setLabel("n1");
		n1.setName("n1");
		//n1.setLayout(null);
		//n2.setMasterSynapse(master);
		n2.setLabel("n2");
		n2.setName("n2");
		//n2.setLayout(null);

		PLPotential p1 = new PLPotential(50);
		p1.setName("p1");
		n1.setPotential(p1);
		PLPotential p2 = new PLPotential(50);
		p2.setName("p2");
		n2.setPotential(p2);

		ConstantPLThreshold t1 = new ConstantPLThreshold();
		ConstantPLThreshold t2 = new ConstantPLThreshold();
		t1.setName("t1");
		n1.setThreshold(t1);
		t2.setName("t2");
		n2.setThreshold(t2);
		ConstantPLThreshold.setAbsoluteRefractionTime(1.1f);
		ConstantPLThreshold.setQuiescentValue(5);
		ConstantPLThreshold.setRelativeRefractionStartValue(5);
		ConstantPLThreshold.setRelativeRefraction(new PLFunction(0));
		ConstantPLThreshold.calculateRelativeRefractionTime();

		ConstantPLSynapse sy1 = new ConstantPLSynapse(n1, n2, 2.3f, 1.5f);
		sy1.setLabel("sy1");
		ConstantPLSynapse sy2 = new ConstantPLSynapse(n2, n1, 1.45f, 8);
		sy2.setLabel("sy2");

		PLFunction synapseResponse = new PLFunction(3);
		synapseResponse.gradients[0] = 3.5;
		synapseResponse.gradientTimes[0] = 0;
		synapseResponse.gradients[1] = -2;
		synapseResponse.gradientTimes[1] = 1;
		synapseResponse.gradients[2] = 0;
		synapseResponse.gradientTimes[2] = synapseResponse.getEndTime(0, 2);
		System.out.println("Synapse response function set to:");
		System.out.print(synapseResponse.toString());
		ConstantPLSynapse.setResponse(synapseResponse);

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(n1);
		this.add(n2);

		views[0] = new SpikeView("s1");
		views[1] = new SpikeView("s2");
		views[2] = new PLNeuronInternalView("v3", true, true);
		views[3] = new PLNeuronInternalView("v4", true, true);
		for (int i=0; i<4; i++) {
			views[i].setName("v" + (i+1));
			views[i].setTMax(50);
			views[i].setCurrentTimePosition(0.5);
			sync.addView(views[i]);
			this.add(views[i]);
			if (views[i] instanceof PLNeuronInternalView) {
				views[i].setYMin(-1);
				views[i].setYMax(15);
			}
		}

		n1.addOutput((SpikeView) views[0]);
		n2.addOutput((SpikeView) views[1]);
		n1.getPotential().addVariableChangeListener((PLNeuronInternalView) views[2]);
		n1.getThreshold().addVariableChangeListener((PLNeuronInternalView) views[2]);
		n2.getPotential().addVariableChangeListener((PLNeuronInternalView) views[3]);
		n2.getThreshold().addVariableChangeListener((PLNeuronInternalView) views[3]);
	}

	public void init() {
		// start the simulation with the first event
		n1.getInputSynapses()[0].spikeEvent(null);
	}

	public void reset() {
		super.reset();
		init();
	}
}
