/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: SpikeView.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.5 $
 *  Last changed at: $Date: 2002/05/16 20:43:35 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron.visualisation;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import mosaic.sim.kernel.*;
import mosaic.sim.neuron.*;
import mosaic.sim.neuron.visualisation.*;

/** This class implements a scrolling spike view where spikes are simply drawn
 *  as vertical lines marking the simulation time points when the spikes
 *  occured. This class can directly receive spike events (it implements
 *  NeuronInput) and will also record those spike events for drawing and
 *  logging.
 */
public class SpikeView extends ScrollingView implements NeuronInput {

	/** The vertical starting position for the vertical lines indicating the
	 *  spikes.
	 */
	protected double spikePosition = 0;
	/** The height for the vertical lines indicating the spikes. */
	protected double spikeHeight = 10;

	/** The color in which the spikes are painted. */
	protected Color spikeColor = null;

	/** The spike events that occured in the past. */
	protected LinkedList spikes = new LinkedList();

	/** Creates a SpikeView with the given title. */
	public SpikeView(String title) {
		this(title, Color.blue);
	}

	/** Creates a SpikeView with the given title and painting the spikes in the
	 *  given color.
	 */
	public SpikeView(String title, Color spikeColor) {
		this.spikeColor = spikeColor;
		this.setBackground(Color.black);
		/*this.setBorder(BorderFactory.createTitledBorder(
			   BorderFactory.createLineBorder(Color.green), title));*/
	}

	/** Overrides the default method and paints the spikes received in the past
	 *  as vertical lines.
	 *  @see SpikeView#spikes
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		ListIterator iterator = spikes.listIterator();

		g.setColor(spikeColor);
		while (iterator.hasNext()) {
			double time = ((Double) iterator.next()).doubleValue();
			if (time >= tmin && time <= tmax) {
				Point from = worldToGraphics(time, spikePosition);
				Point to = worldToGraphics(time, spikeHeight);
				g.drawLine((int) from.getX(), (int) from.getY(),
					(int) to.getX(), (int) to.getY());
			}
		}
	}

	/** This method is the implementation of the NeuronInput interface and is
	 *  used to receive spikes. A received spike is stored in the list and the
	 *  view is updated.
	 *  @see #spikes.
	 */
	public void spikeEvent(NeuronOutput sender) {
		double currentTime = SimApplication.getCurrentSimApplication().getTime();
		spikes.add(new Double(currentTime));
		updateView();
	}
}
