/* Generated by Together */

package mosaic.sim.util.thread;

import java.util.LinkedList;
import concurrent.ReadWriteLock;
import concurrent.WriterPreferenceReadWriteLock;

/** The basic implementation of a ThreadEventDispatcher. */
public class BasicThreadEventDispatcher implements ThreadEventDispatcher {

    /** The run mode. Control variable for the eventLoop() method. */
    protected boolean run=true;

    /** The monitor for the run variable. */
    protected Integer syncRun=new Integer(0);

    /** The eventlist. All new events will be placed in this list. */
    protected LinkedList eventList=new LinkedList();

    /** This object will be notified when a new event object is inserted. */
    protected Object notifyObject=this;

    /** Sets the current runmode. Used to control the eventLoop() method. */
    public final void setRun(boolean run){
        synchronized(syncRun){
            this.run=run;
        }
    }

    /** Returns the current run mode. Used to control the eventLoop() method. */
    public final boolean isRun(){
        boolean currentRun;
        synchronized(syncRun){
            currentRun=run;
        }
        return currentRun;
    }

    /** Sets the notify object, that will be notified when a new event is inserted. */
    public final void setNofiyObject(Object notifyObject){
        if (notifyObject==null){
            this.notifyObject=this;
        }else{
            this.notifyObject=notifyObject;
        }
    }

    /** Returns the notify object. */
    public final Object returnNotifyObject(){
        return notifyObject;
    }

    /** Adds a new EventWrapper to the event list and notifies the sleeping thread. */
    public void add(EventWrapper ew){
        synchronized(eventList){
            eventList.add(ew);
        }
        synchronized(notifyObject){
            notifyObject.notify();
        }
    }

    /** Processes exactly one event object. Returns false if the event list was empty. */
    public final boolean processOne(){
        EventWrapper ew=null;
        synchronized(eventList){
            if (eventList.size()>0){
                ew=(EventWrapper)eventList.removeFirst();
                ew.deliver();
            }
        }
        return ew!=null;
    }

    /** Processes all events on the event list. */
    public void processAll(){
        while (processOne()){};
    }

    /** Implements an event loop, which loops all the time. */
    public final void eventLoop(){
        while (isRun()){
            processAll();
        }
    }

    /** Returns false if new events are waiting for execution. */
    public boolean isEmpty(){
        return eventList.isEmpty();
    }
}
