
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class TwoDimHashtable          ----------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------




package mosaic.util;

import java.util.*;

/**
* Class TwoDimHashtable implements a Hashtable which contains Hashtable as values
* which contain the elements.
* Use this class to realize a two dimensional mapping of two keys to a value.
*/
public class TwoDimHashtable extends Hashtable {

  public TwoDimHashtable() {
  }

  public TwoDimHashtable(int i) {
      super(i);
  }

  public TwoDimHashtable(int i, float f) {
      super(i, f);
  }

  /** Gets the Enumeration of all values in the TwoDimHashtable. */
  public   Enumeration elements () {
      Vector allElements = new Vector();
      Enumeration keys = keys();
      while (keys.hasMoreElements())  {
          Enumeration elems = elements(keys.nextElement());
          while (elems.hasMoreElements()) {
              allElements.addElement(elems.nextElement());
          }
      }
      return allElements.elements();
  }

  /** Gets the Enumeration of values for that key. */
  public   Enumeration elements (Object key) {
      Hashtable h = getHashtable(key);
      if (h == null) {
          h = new Hashtable();
      }
      return h.elements();
  }

    /** Gets the vector containing the elements for that key.*/
    public   Hashtable getHashtable (Object key) {
      Object val = get(key);
      if (val != null) {
          Hashtable h;
          try {
            h = (Hashtable)val;
          }
          catch (ClassCastException e) {
            h = null;
          }
            return h;
      }
      else
          return null;
  }

  /** Tests if the value is contained in any vector for any key. */
  public   boolean contains (Object value) {
      Enumeration keys = keys();
      boolean notFound = true;
      while (keys.hasMoreElements() && notFound)  {
          if (contains(keys.nextElement(), value)) {
              notFound = false;
              return true;
          }
      }
      return false;
  }

  /** Tests if the value is contained in any vector for any key. */
  public   boolean contains (Object key, Object value) {
      Hashtable h = getHashtable(key);
      if (h != null)
          return h.contains(value);
      else
          return false;
  }


  /** Adds a value to the vector of values for that key. */
  public   Object get (Object key1, Object key2) {
      Hashtable h = getHashtable(key1);
      if (h == null)
          return null;
      else
          return h.get(key2);
  }

  /** Adds a value to the vector of values for that key. */
  public   Object put (Object key1, Object key2, Object value) {
      Hashtable h = getHashtable(key1);
      if (h == null) {
          h = new Hashtable();
          super.put(key1, h);
      }
      return h.put(key2, value);
  }


  /** Removes  a value from the vector of values for that key. */
  public   Object remove (Object key1, Object key2) {
      Hashtable h = getHashtable(key1);
      if (h != null)
            return h.remove(key2);
      else
          return null; 
  }

}
