package mosaic.util.custom;

import java.beans.*;
import java.awt.*;

/**
 * The ArrayEditor class supports a custom Editor, which makes it possible
 * to edit Arrays with Components of any Class or primitive Type in a visual
 * way. A Propertyeditor for the componenttype of the array should exist.
 * <p>
 * PLEASE MAKE SURE THAT THE GIVEN OBJECT (after Instantiation of objects
 * of this class with setValue) IS NOT NULL AND AN ARRAY.
 * i.e. an Object[0].
 * Otherwise the component type cannot be determined.
 *
 * @version 17 Feb 1999
 * @author Thomas Hilpold
 */
public class ArrayEditor extends PropertyEditorSupport
        implements CustomArrayEditorAdapter {

    //
    // Controlling output for debugging purposes */
    //

    private boolean[] DEBUG = new boolean[] {true};


    //
    // PropertyEditorSupport Methods
    //

    public ArrayEditor() {
      if (DEBUG[0]) OutFrame.out(this.toString() + " CREATED");
    }

    public String getAsText () {
        OutFrame.out(this.toString() + " GETASTEXT : " + super.getValue());
        return super.getAsText();
    }

    public Object getValue() {
        OutFrame.out(this.toString() + " GETVALUE : " + super.getValue());
        return super.getValue();
    }

    public void setValue(Object value) {
        Object oldVal;
        OutFrame.out(this.toString() + " SETVALUE : " + value);
        /* set and fire only, if new Value differs from current */
        oldVal = super.getValue();
        if (    ((value != null) && (value.equals(oldVal)))
             || ((value == null) && (oldVal == null))
           ) {

            /* they are the same - donothing */
        }
        else
            super.setValue(value);
    }

    public void setAsText(String text)
            throws java.lang.IllegalArgumentException {
        OutFrame.out(this.toString() + " SETASTEXT FAIL!" + text);
        //not possible
        throw new java.lang.IllegalArgumentException();
    }

    public String[] getTags()
    {
        return null;
    }

    /**
     * Syntax:
     * new Integer[]{new Integer(3) , ...};
     */
    public String getJavaInitializationString() {
      OutFrame.out(this.toString() + " GETINIT : "+ super.getValue());
      return (new InitStringCreator().getInitStringForArray(super.getValue()));
    }

    public boolean supportsCustomEditor() {
        return true;
    }

    /** creates and returns a CustomArrayPropEditor */
    public java.awt.Component getCustomEditor() {
        if (DEBUG[0]) OutFrame.out(this.toString() + "  getCustomEd with: "
                + super.getValue());
        if (super.getValue() != null) {
            Component ret;
            try {
                ret = new CustomArrayPropEditor(this);
            } catch (Exception e) {
                OutFrame.out(this.toString() + "  :" + e.getMessage());
                ret = new Label("COULD NOT CREATE CUSTOMEDITOR");
            };
            return ret;
        }
        else {
            if (DEBUG[0]) OutFrame.out(this.toString()
                    + "VALUE NULL & CUSTOMEDIT!");
            return new Label("ARRAY WAS NULL - CANNOT EDIT!");
        }
    }

    //
    // PropertyChangeSupport Methods
    //

    public void addPropertyChangeListener(PropertyChangeListener listener) {
      if (DEBUG[0]) OutFrame.out(this.toString() +
             "  Add PropertyChangeListener " + listener);
      super.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
      if (DEBUG[0]) OutFrame.out(this.toString() +
            "  remove PropertyChangeListener" + listener);
      super.removePropertyChangeListener(listener);
    }

    public void firePropertyChange() {
      if (DEBUG[0]) OutFrame.out(this + "fire ! ");
      super.firePropertyChange();
    }

    //
    // CustomArrayEditorAdapter Methods
    //

    public void setAsArray(Object arr) {
        if (DEBUG[0]) OutFrame.out(this.toString() + " setAsArray :" + arr);
        super.setValue(arr);
    }

    public Object getAsArray() {
        Object ret;
        ret = super.getValue();
        if (DEBUG[0]) OutFrame.out(this.toString() + " getAsArray: " + ret);
        return ret;
    }

}
