package mosaic.util.custom;

import javax.swing.table.*;
import java.lang.reflect.*;

/**
 * TableModel to be used by CustomArrayPropEditor.
 *
 * @version 17 Feb 1999
 * @author Thomas Hilpold
 * @see CustomArrayPropEditor
 */
public class CustomArrayTableModel extends AbstractTableModel {

    /** creates a new Array which is length + 1 */
    public void addIndex() {
        Object[] newData = (Object[])Array.newInstance(data.getClass().getComponentType(), data.length +1);
        for (int i = 0; i < data.length; i++)
            newData[i] = data[i];
        newData[newData.length - 1] = null;
        if (DEBUG[0]) OutFrame.out("new Array - Type: " + newData.toString());
        data = newData;
        fireTableRowsInserted(data.length -1, data.length -1);
    }

    /** creates a new Array without the last line. */
    public void delIndex() {
        if(data.length > 0) {
            Object[] newData = (Object[])Array.newInstance(data.getClass().getComponentType(), data.length - 1);
            for (int i = 0; i < newData.length; i++)
                newData[i] = data[i];
            data = newData;
            fireTableRowsDeleted(data.length, data.length );
        }
    }

    /** creates a new Array without the given index */
    public void delIndex(int index) {
        if ((index >= 0) && (index < data.length) && (data.length >= 1)) {
            Object[] newData = (Object[])Array.newInstance(data.getClass().getComponentType(), data.length - 1);
            for (int i = 0; i < newData.length; i++) {
                if (i < index) newData[i] = data[i];
                else if (i >= index) newData[i] = data[i + 1];
            }
            data = newData;
            fireTableRowsDeleted(index , index );
        }
    }

    /** sets a new Array */
    public void setData(Object[] newData) {
        data = newData;
        fireTableStructureChanged();
    }

    /** returns the Array*/
    public Object[] getData() {
       return data;
    }

    /** returns index or value of index */
    public Object getValueAt(int row, int col) {
        if (col == 0) return new Integer(row);
        else return data[row];
    }

    /** Sets only if data type is ok or null. */
    public void setValueAt(Object val, int row, int col) {
        if ((DEBUG[0]) && (val != null)) OutFrame.out(this.toString()
                + "Value Type = " + val.getClass().toString());
        if ((col == 1) && (row < data.length)) {
            if (val != null) {
                //check if assign allowed
                Class comType = data.getClass().getComponentType();
                if (comType.isInstance(val)) {
                    data[row] = val;
                }
                else
                    System.err.println("Tried to set wrong type !");
            }
            else {
                //null can be assigned anytime
                data[row] = null;
            }
            shallowCopyData();
            fireTableCellUpdated(row, col);
        }
        //else - invalid array field row,col
    }

    /** returns number of rows*/
    public int getRowCount() {
        return data.length;
    }

    /** returns 2 (index and values) */
    public int getColumnCount() {
        return 2;
    }

    public String getColumnName(int col) {
        if (col == 0) return ("Index");
        else return data.getClass().getComponentType().toString();
    }

    public Class getColumnClass(int col) {
        if (col == 0)
            return Integer.class;
        else
            return data.getClass().getComponentType();
    }

    /** only index is not editable */
    public boolean isCellEditable(int row, int col) {
        if (col > 0) return true;
        else return false;
    }

    /** creates a new Array with same size and copies the references */
    private void shallowCopyData() {
        Object[] newData;
        newData = (Object[])Array.newInstance(data.getClass().getComponentType(), data.length);
        for (int i = 0; i < data.length; i++)
            newData[i] = data[i];
        data = newData;
    }

    //
    // Data
    //

    private Object[] data;

    //
    // Debugging output
    //

    private boolean[] DEBUG = new boolean[] {false, false};
}
