package  mosaic.util.custom;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.EventObject;
import javax.swing.*;

/**
 * Instances contain a Propertyeditor with a CustomEditor. A Dialog with
 * the CustomEditor will be opened, when the user clicks on the cell.
 * <p>
 * Uses code of Sun DefaultCellEditor with modifications!
 * I had problems to inherit and modify the delegate.
 * Used by CustomArrayCellEditor as a Delegate.
 *
 * @version 17 Feb 1999
 * @author Thomas Hilpold
 * @see CustomArrayCellEditor
 */
public class CustomCellEditor extends DefaultCellEditor implements ActionListener{

   public CustomCellEditor(PropertyEditor aPEWithCPE) {
        super(new JCheckBox()); //unfortunately no Defaultconstructor
        if (aPEWithCPE.supportsCustomEditor()) {
            this.pe = aPEWithCPE;
            if (pe.getValue() != null)
                this.editorComponent = new JButton(pe.getAsText());
            else
                this.editorComponent = new JButton("null");
            //Definition of EditorDelegate
            ((JButton)this.editorComponent).addActionListener(this);
            setClickCountToStart(1);
        }
   }

   public void setValue(Object x) {
        pe.setValue(x);
   }

   public Object getCellEditorValue() {
        return pe.getValue();
   }

   public boolean isCellEditable(EventObject anEvent) { return true;}

   public boolean shouldSelectCell(EventObject anEvent) { return true;}

   public boolean stopCellEditing() {
        fireEditingStopped();
        return true;
   }

   public void cancelCellEditing() {
        fireEditingCanceled();
   }

   public Component getTableCellEditorComponent(JTable jt, Object value,
                                        boolean isSelected, int row, int col) {
        //some editors do not allow null to be set
        if (value != null) setValue(value);
        if (pe.getValue() != null)
            ((JButton)editorComponent).setText(pe.getAsText());
        else
            ((JButton)editorComponent).setText("null");
        return editorComponent;
   }

   /** Shows a Dialog with the CustomEditor of the PropertyEditor */
   public void actionPerformed(ActionEvent e) {
       if (e.getSource() == editorComponent) {
          if (DEBUG[1]) OutFrame.out(this.toString() + "ActionPerformed!");
          JDialog editDlg = new CustomEditorDialog(pe.getCustomEditor(), "CellEditor");
          editDlg.setLocationRelativeTo(editorComponent);
          editDlg.setVisible(true);
          //User will Edit using a PropertySheet in a Dlg
          //which contains an OK-Button to close it
          //after editin we must do...
          setValue(pe.getValue());
          fireEditingStopped();
          if (DEBUG[1]) OutFrame.out(this.toString() + "Dialog closed fired Edit stopped!");
       }
   }

   //
   // Data
   //

   protected PropertyEditor pe;

   //
   // Debugging output
   //

   private boolean [] DEBUG = new boolean[] {false, false};

}


