package mosaic.vis.input;

import javax.swing.*;
import mosaic.sim.kernel.output.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.event.*;
import mosaic.sim.kernel.continuous.*;
import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.simevent.*;
import mosaic.sim.kernel.*;
import mosaic.vis.output.expandable;
import mosaic.vis.output.ExpandSymbol;
import mosaic.vis.input.BoundNumberInputView;
import mosaic.vis.input.SliderInputView;

public class DualInputView extends AbstractModelComponent implements expandable{


    public DualInputView() {
        try  {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    // Views integrated into an expandable view :
    BoundNumberInputView numberInputView = new BoundNumberInputView();
    SliderInputView slider = new SliderInputView();
    ExpandSymbol expandi = new ExpandSymbol();

    private void jbInit() throws Exception {

        slider.getSlider().addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(ChangeEvent e) { jSlider_stateChanged(e); }
        });

/*        expandi.addMouseListener( new MouseAdapter() {
            public void mousePressed(MouseEvent e) { viewHelpWindow(e); }
            public void mouseReleased(MouseEvent e) { disposeHelpWindow(e); }
        });
        numberInputView.getTextField().addMouseListener( new MouseAdapter() {
            public void mousePressed(MouseEvent e) { viewHelpWindow(e); }
            public void mouseReleased(MouseEvent e) { disposeHelpWindow(e); }

        } );

        slider.addMouseListener( new MouseAdapter() {
            public void mousePressed(MouseEvent e) { viewHelpWindow(e); }
            public void mouseReleased(MouseEvent e) { disposeHelpWindow(e); }
       } );
 */

        expandedSize = new Dimension(200,100);
        collapsedSize = new Dimension(40,20);
        numberInputViewBounds = new Rectangle(1,1,collapsedSize.width - 10, collapsedSize.height - 2);
        sliderBounds = new Rectangle(1, numberInputViewBounds.height + 2,
                                     expandedSize.width - 10,
                                     expandedSize.height -
                                     numberInputViewBounds.height - 2);

        this.setSize(collapsedSize);
        slider.setBounds(sliderBounds);
        expandi.setBounds( collapsedSize.width - 10, 1, 5, 10 );

        numberInputView.setBackground(Color.white);
        numberInputView.setBounds(numberInputViewBounds);
        this.setBackground(Color.white);

        slider.setBackground(Color.white);

        expandi.setBackground(Color.white);
        expandi.setForeground(Color.blue);
        expandi.setOwner(this);

        this.add(numberInputView, null);
        this.add(expandi, null);
    }

    private void jSlider_stateChanged(ChangeEvent e) {
        if ( ! ((JSlider)e.getSource()).getValueIsAdjusting() ) {
            collapse();
        }
    }

    // -------------------------------------------------------------------------
    // properties --------------------------------------------------------------
    // -------------------------------------------------------------------------

    // expanding ---------------------------------------------------------------
    boolean expanded = false;
    boolean collapsingOrExpanding = false;
    Dimension collapsedSize, expandedSize;
    Rectangle sliderBounds, numberInputViewBounds;

    public boolean isExpanded() { return expanded; }

    public void collapse() {
        // do not change representation if Simulation is in HelpMode
//        if (!isHelpMode()) {
            if (expanded) {
                collapsingOrExpanding = true;
                numberInputView.setBounds(numberInputViewBounds);
                this.remove(slider);
                this.add(numberInputView);
                this.add(expandi);
                expandi.setBounds( collapsedSize.width - expandi.getWidth()-2,
                                   1,
                                   expandi.getWidth(),
                                   expandi.getHeight() );
                this.setSize(collapsedSize);
                collapsingOrExpanding = false;
                expanded = false;
            }
//        }
    }

    public void expand() {
        // do not change representation if Simulation is in HelpMode
//        if (!isHelpMode()) {
            if (!expanded) {
                collapsingOrExpanding = true;
                slider.setBounds(sliderBounds);
                numberInputView.setBounds(numberInputViewBounds);
                this.remove(numberInputView);
                this.add(slider);
                this.add(numberInputView);
                this.add(expandi);
                expandi.setBounds( expandedSize.width - expandi.getWidth()-2,
                                   1,
                                   expandi.getWidth(),
                                   expandi.getHeight() );
                this.setSize(expandedSize);
                collapsingOrExpanding = false;
                slider.revalidate();
                expanded = true;
            }
//        }
    }


    // properties defining range for slider and inputView ----------------------

    public void setMax(double m) {
        numberInputView.setMaxVal(m);
        slider.setMaximum((int)m);
    }

    public double getMax() {
        return numberInputView.getMaxVal();
    }

    public void setMin(double m) {
        numberInputView.setMinVal(m);
        slider.setMinimum((int)m);
    }

    public double getMin() {
        return numberInputView.getMinVal();
    }

    // properties delegated to slider
    public void setMinorTickSpacing(int m) {
        slider.setMinorTickSpacing(m);
    }

    public int getMinorTickSpacing() {
        return slider.getMinorTickSpacing();
    }

    public void setMajorTickSpacing(int m) {
        slider.setMajorTickSpacing(m);
    }

    public int getMajorTickSpacing() {
        return slider.getMajorTickSpacing();
    }

    public void setInitValue(int i) {
        slider.setInitValue(i);
    }

    public int getInitValue() {
        return slider.getInitValue();
    }

    public void setPaintTicks(boolean b) {
        slider.setPaintTicks(b);
    }

    public boolean getPaintTicks() {
        return slider.getPaintTicks();
    }

    // property font, forward fontsettings to numberInputView and slider
    public void setFont(Font f) {
        numberInputView.setFont(f);
//        slider.getJLabel().setFont(f);
        numberInputView.revalidate();
    }

    /** sets tooltipText of textField */
    public void setToolTipText(String txt) {
         numberInputView.setToolTipText(txt);
         slider.setToolTipText(txt);
         super.setToolTipText(txt);
    }

    // properties defining size of numberInputView and Slider ------------------
    // expect TrendView to be always larger than numberView, -------------------
    // if not ignore settings --------------------------------------------------

    public Dimension getExpandedSize() {
        return expandedSize;
    }

    public void setExpandedSize(Dimension size) {
        expandedSize.setSize(size);
        sliderBounds = new Rectangle(1, numberInputViewBounds.height + 2,
                                     expandedSize.width - 10,
                                     expandedSize.height -
                                     numberInputViewBounds.height - 2);

    }


    public void setBounds( int x, int y, int w, int h) {
        // collapsedSize
        if (!collapsingOrExpanding)
        {  collapsedSize.setSize(w,h);
           numberInputViewBounds.setBounds(1, 1, w-10, h-2);
           numberInputView.setBounds(numberInputViewBounds);
           expandi.setBounds( w - expandi.getWidth() - 2 , 1,
                              expandi.getWidth(), expandi.getHeight() );
           sliderBounds = new Rectangle(1, numberInputViewBounds.height + 2,
                                        expandedSize.width - 10,
                                        expandedSize.height -
                                        numberInputViewBounds.height - 2);
        }
        super.setBounds(x,y,w,h);
    }

    public boolean isOptimizedDrawingEnabled() { return false; }
    // property observedVariable -----------------------------------------------
    UserEventListener userEventListener;
    DoubleVariable observed;

    public void setUserEventListener(UserEventListener l) {
        if (userEventListener != null) {
        // remove trend as variableChangeListener from old observed variable
            userEventListener.removeUserEventComponent(numberInputView);
            userEventListener.removeUserEventComponent(slider);
        }
        userEventListener = l;
        numberInputView.addUserEventListener(l);
        slider.addUserEventListener(l);
    }

    public void removeUserEventListener(UserEventListener l) {
        l.removeUserEventComponent(numberInputView);
        l.removeUserEventComponent(slider);
        userEventListener = null;
    }

    public void setObservedVariable(DoubleVariable v) {
        if (observed != null) {
            v.removeVariableChangeListener(numberInputView);
            v.removeVariableChangeListener(slider);
        }
        v.addVariableChangeListener(numberInputView);
        v.addVariableChangeListener(slider);
    }

    public void removeObservedVariable(DoubleVariable v) {
        v.removeVariableChangeListener(numberInputView);
        v.removeVariableChangeListener(slider);
    }

    public UserEventListener getUserEventListener() {
        return userEventListener;
    }

    // -------------------------------------------------------------------------
    // paintComponent ----------------------------------------------------------

    public void paintComponent(Graphics g) {
        g.setColor(getBackground());
        g.fillRect(0,0, getWidth(), getHeight());
        g.setColor(getForeground());
        g.drawRect(0,0,getWidth()-1, getHeight()-1);
    }

    // setBackground, delegates backgroundcolor to all child components
    public void setBackground(Color c) {
        numberInputView.setBackground(c);
        slider.setBackground(c);
        expandi.setBackground(c);
        super.setBackground(c);
    }

}
