//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class BoundNumberInputView    -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.vis.input;

/**
* Animation view to visualize and allow input of numeric values.
* Can serve as variable change listeners for numeric variables.
* Can serve as userInputSource for numeric values
* Displays the number in a textfield.
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* <li> set minimum input value by <code>setMinVal</code>
* <li> set maximum input value by <code>setMaxVal</code>
** </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/

public class ScaledBoundNumberInputView extends BoundNumberInputView {

  /** Default constructor */
  public ScaledBoundNumberInputView() {
    super();
  }

  /** reads a numeric value from text and returns it as Object, divides user
      value by scaleFactor */
  public Object readValue(String text) {
    try
    { Double value = new Double(text);
      if (value.doubleValue() < (minVal*scaleFactor))
        return new Double(minVal/scaleFactor);
      else if (value.doubleValue() > (maxVal*scaleFactor))
        return new Double(maxVal/scaleFactor);
      else
        return (new Double( value.doubleValue() / scaleFactor));
    }catch (NumberFormatException e)
    { return null;
    }
  }

  public void setText(String text) {
      if (getNumber() == null) {
          super.setText(text);
      }
      else {
          // ignore text, and call super.setText with new generated text
          super.setText( (new Double(getNumber().doubleValue()*scaleFactor)).toString());
      }
  }


// propertie scaleFactor -------------------------------------------------------
  double scaleFactor = 1.0;

  /**
  * Sets the new scaleFactor, only employed directly before painting value or
  * after reading value from user
  */
  public void setScaleFactor(double s) {
    // read min and max with old scalingFactor
    double min = getMinVal();
    double max = getMaxVal();
    scaleFactor = s;
    // set min and max again with new scalingFactor
    setMinVal(min);
    setMaxVal(max);
    if (getNumber() != null)
      setValue(getNumber());        
  }

  /**
  * Gets the actual scaleFactor.
  */
  public double getScaleFactor() {
    return scaleFactor;
  }


  // overwrite property getter und setter to consider scaleFactor
  public void setMinVal(double newMin) {
      super.setMinVal(newMin/scaleFactor);
  }

  /**
  * Gets the actual minimum for values entered.
  */
  public double getMinVal() {
    return super.getMinVal()*scaleFactor;
  }


  /**
  */
  public void setMaxVal(double newMax) {
      super.setMaxVal(newMax/scaleFactor);
  }

  /**
  * Gets the actual minimum for values entered.
  */
  public double getMaxVal() {
      return super.getMaxVal()*scaleFactor;
  }



}
