//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger, Bernhard Reitinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class NumberInputView -------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.vis.input;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import mosaic.vis.input.ui.*;

/**
* Animation view to visualize and allow input of numeric values.
* Can serve as variable change listeners for numeric variables.
* Can serve as userInputSource for numeric values
* Displays the number in a textfield.
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @author Bernhard Reitinger (redesigned !) 08/03/2000
* @version %I% %G%
*/

public class SliderInputView extends BasicNumberInputView {
    JSlider jSlider = new JSlider();
    BorderLayout borderLayout1 = new BorderLayout();
    private SliderInputUI sliderUI = new SliderInputUI(jSlider,this);

    /** Default constructor */
    public SliderInputView() {
        super();
        try  {
 	    jSlider.setUI(sliderUI);
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
	this.setLayout(borderLayout1);
        jSlider.addMouseListener(new MouseAdapter() {
		public void mouseReleased(MouseEvent e) {
		    setValue(new Double(sliderUI.getValueStr()));
		    triggerEvent();
		}
	    });
 	this.add(jSlider);
    }

    // Overwrite paintGfx to avoid painting of background rectangle and
    // label
    public void paintWithGfx(Graphics g) {
    }

    // returns slider
    public JSlider getSlider() { return jSlider; }

    // Properties forwarded to jslider
    public void setExtent(int ext)         { jSlider.setExtent(ext);           }

    public int getExtent()                 { return jSlider.getExtent();       }

    public void setMajorTickSpacing(int n) { jSlider.setMajorTickSpacing(n);   }

    public int getMajorTickSpacing()       { return jSlider.getMajorTickSpacing(); }

    public void setMaximum(int max) { 
	jSlider.setMaximum(max);
    }

    public int getMaximum()                { return jSlider.getMaximum();      }

    public void setMinimum(int min) {
	jSlider.setMinimum(min);
    }

    public int getMinimum()                { return jSlider.getMinimum();      }

    public void setMinorTickSpacing(int n) { jSlider.setMinorTickSpacing(n);   }

    public int getMinorTickSpacing()       { return jSlider.getMinorTickSpacing(); }

    public void setOrientation(int o)      { jSlider.setOrientation(o);        }

    public int getOrientation()            { return jSlider.getOrientation();  }

    public void setPaintLabels(boolean b)  { jSlider.setPaintLabels(b);        }

    public boolean getPaintLabels()        { return jSlider.getPaintLabels();  }

    public void setPaintTicks(boolean b)   { jSlider.setPaintTicks(b);         }

    public boolean getPaintTicks()         { return jSlider.getPaintTicks();   }

    public void setPaintTrack(boolean b)   { jSlider.setPaintTrack(b);         }

    public boolean getPaintTrack()         { return jSlider.getPaintTrack();   }

    public void setSnapToTicks(boolean b)  { jSlider.setSnapToTicks(b);        }

    public boolean getSnapToTicks()        { return jSlider.getSnapToTicks();  }

    public void setInitValue(int val)      { jSlider.setValue(val);
                                             setValue(new Double(val));
                                  		     triggerEvent();
                                           }

    public int getInitValue()              { return jSlider.getValue();        }


    public static void main(String[] args) {
	javax.swing.JFrame f = new javax.swing.JFrame();
  	SliderInputView s = new SliderInputView();
// 	JSlider s = new JSlider();
	javax.swing.JPanel panel = new javax.swing.JPanel();

// 	System.out.println(s.getUI());
	panel.setLayout(null);
	s.setMajorTickSpacing(10);
	s.setPaintTicks(true);
	s.setLocation(0,0);
	s.setMinimum(-10);
	s.setInitValue(0);
	
	s.setBounds(30,0,200,100);
//  	s.setCondInput(true);
	panel.add(s);
	f.getContentPane().add(s);
	f.setSize(500,100);
	f.setLocation(100,100);
	f.show();
	
    }


}
