package mosaic.vis.output;

import java.awt.*;
import mosaic.sim.kernel.variable.*;

// Base class for flowViews, only provides Property flowColor
public class BarView extends ContVariableView {

    public final static int LEFT_RIGHT = 1;
    public final static int RIGHT_LEFT = 2;
    public final static int BOTTOM_TOP = 3;
    public final static int TOP_BOTTOM = 4;
    public final int BORDERWIDTH = 2;

    public BarView() {
    }

    /** true...forward   false...backward */
    int direction = LEFT_RIGHT;

     /**
       * Get the direction of the bar.
       * @return Value of direction.
       */
    public int getDirection() {return direction;}

    /**
       * Set the value of alignment.
       * @param v  Value to assign to alignment.
       */
    public void setDirection(int  dir) {
        this.direction = dir;
    }
    
    // Property flowColor
    Color flowColor = Color.blue;

    public void setFlowColor( Color c) {
        flowColor = c;
    }

    public Color getFlowColor () {
        return flowColor;
    }

    public void paintValue(Graphics g) {
        // paint background
        g.setColor(getBackground());
        g.fillRect(0, 0, getWidth()-1, getHeight()-1 );
        //paint bar
        double val = Math.min(this.getDoubleValue(), max);
        g.setColor(getFlowColor());
        if (direction == LEFT_RIGHT || direction == RIGHT_LEFT) {
            double exactBarWidth = (getWidth()-2*BORDERWIDTH) * ((val-min)/(max-min));
            int barWidth;
            if (exactBarWidth < (max-min)/2) {
                //round up
                barWidth = ((int)(Math.ceil(getWidth()-2*BORDERWIDTH) * ((val-min)/(max-min))));
            } else {
                barWidth = ((int)(Math.rint(getWidth()-2*BORDERWIDTH) * ((val-min)/(max-min))));
            }
            // correct barWidth for small values
            if (barWidth == 0 && val > min+(max-min)/1000) {
                barWidth = 1;
            }
            if (direction == LEFT_RIGHT) {
                g.fillRect(BORDERWIDTH, 0, barWidth, getHeight() );

            }
            else if (direction == RIGHT_LEFT) {
                g.fillRect(getWidth()-barWidth-BORDERWIDTH, 0, barWidth, getHeight() );
            }
        } else if (direction == BOTTOM_TOP || direction == TOP_BOTTOM) {
            double exactBarHeight = (getHeight()-2*BORDERWIDTH) * ((val-min)/(max-min));
            int barHeight;
            if (exactBarHeight < (max-min)/2) {
                //round up
                barHeight = ((int)(Math.ceil(getHeight()-2*BORDERWIDTH) * ((val-min)/(max-min))));
            } else {
                barHeight = ((int)(Math.rint(getHeight()-2*BORDERWIDTH) * ((val-min)/(max-min))));
            }
            // correct barWidth for small values
            if (barHeight == 0 && val > min+(max-min)/1000) {
                barHeight = 1;
            }
            if (direction == TOP_BOTTOM) {
                g.fillRect(0, BORDERWIDTH, getWidth(), barHeight);

            }
            else if (direction == BOTTOM_TOP) {
                g.fillRect(0, getHeight()-barHeight-BORDERWIDTH, getWidth(), barHeight );
            }
        }
        g.setColor(getForeground());
        g.drawRect(0,0, getWidth() - 1, getHeight() -1 );
        g.drawRect(1,1, getWidth() - 3, getHeight() -3 );

    }

}
