
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class BooleanView        ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.vis.output;

import java.awt.*;
import java.util.*;
import java.beans.*;
import mosaic.sim.kernel.*;

/**
* Animation view to visualize boolean values.
* If true, paints the rectangle in foreground color and displays onText (T).
* If false, paints the rectangle in background color and displays offText (F).
* <p>
* Use the view by adding it as VariableChangeListener to a boolean variables.
* <p>
* <b>Costumization:</b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* <li> set the text to display if true by  <code>setOnText</code>
* <li> set the text to display if false by  <code>setOffText</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/
public class BooleanView extends TextView {

    /** Default constructor BooleanView */
  public BooleanView() {
    setOnOff(false);
    setLabel(""); 
  }

// boolean value ---------------------------------------------------------------------------------------------

    /** variable to store the boolean property value */
  boolean onOff= false;
    /** Sets a new boolean value to display */
  public void setOnOff(boolean newValue) {
    onOff= newValue;
      if (onOff) setText(onText);
      else setText(offText);
      repaint();
  }
    /** Gets the current boolean value displayed */
  public boolean getOnOff() {
    return onOff;
  }

// on/off text -----------------------------------------------------------------------------------------------
  
    /** variable to store the text displayed when true */
  private String onText= "T";
  /** Gets the text to be displayed when true */
  public String getOnText() { return onText; }
  /** Sets the text to be displayed when true */
  public void setOnText(String htext) { onText= htext; }

  /** variable to store the text displayed when false */
  private String offText= "F";
  /** Gets the text to be displayed when false */
  public String getOffText() { return offText; }
  /** Sets the text to be displayed when false */
  public void setOffText(String htext) { offText= htext; }


// setValue --------------------------------------------------------------------------------------------------

    /**
    * Reacts to a new property value by setting the onOff value.
    * Called by super.propertyChange.
    * @param newVal The new value object received from the property change
    */
    public void setValue (Object newVal) {
        if (newVal instanceof Boolean) {
            super.setValue(newVal);
            setOnOff (((Boolean)newVal).booleanValue());
        }
        else
            System.out.println(">> tried to send non Boolean value to " + toString());
    }

// paintValue ------------------------------------------------------------------------------------------------

    /**
    * Paints the boolean value.
    * @param g the graphics context of the view
    */
    public void paintValue(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    if (onOff) {
        g.setColor(getForeground());
        g.fillRect(0,0,width,height);
        g.setFont(getFont());
        g.setColor(getBackground());
        FontMetrics fm = g.getFontMetrics();
        g.drawString(getText(), (width - fm.stringWidth(getText())) / 2,
          (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
    }
    else {
        g.setColor(getBackground());
        g.fillRect(0,0,width,height);
        g.setFont(getFont());
        g.setColor(getForeground());
        FontMetrics fm = g.getFontMetrics();
        g.drawString(getText(), (width - fm.stringWidth(getText())) / 2,
          (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
    }
  }

}

