
/**
* Animation view to visualize numeric values.
* Can serve as variable change listeners for numeric variables.
* Displays the number using drawString.
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/

package mosaic.vis.output;
import java.awt.*;
import java.util.*;
import java.beans.*;


public class FormattedNumberView extends NumberView{

    /** Default constructor NumberView */
    public FormattedNumberView() {
    }
    // paintValue ------------------------------------------------------------------------------------------------

    /**
    * Paints the number value;
    * by putting out the text representing the number in the view, formatted to
    * nrDigits after comma.
    * @param g     the graphics context of the view
    */
    public void paintValue(Graphics g) {
       int width = getSize().width;
       int height = getSize().height;
       g.setFont(getFont());
       g.setColor(getForeground());
       FontMetrics fm = g.getFontMetrics();
       // is number less than visible range (10 E-digits) ?
       String outText;
       if (( getNumber() == null) ||
           ( number.doubleValue() < (Math.pow(10, 1 - fractionDigits))) ){
           outText = decimalFormat.format(0.0);
       } else {
           outText = decimalFormat.format(number.doubleValue()) + unitString;
       }
       g.drawString( outText,
                   (width - fm.stringWidth(outText)) / 2,
                   (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
     }

     // Properties

     // digits -----------------------------------------------------------------
     int fractionDigits = 2;
     int intDigits = 4;
     java.text.DecimalFormat decimalFormat = new java.text.DecimalFormat ("###0.0#");

     private String getFormatString() {
        String st = new String();
        for (int i = 1; i<intDigits; i++)  st = st + "#";
        if (intDigits > 0) st = st + "0";
        if (fractionDigits > 0) st = st + ".0";
        for (int i = 1; i<fractionDigits; i++) st = st + '#';

        return st;
     }

     /**
     * Sets the new number of digits to be draw in view after comma.
     */
     public void setFractionDigits(int d) {
       fractionDigits = d;
       decimalFormat = new java.text.DecimalFormat (getFormatString());
     }

     /**
     * Gets number of fraction digits
     */
     public int getFractionDigits() {
       return fractionDigits;
     }

      /**
     * Sets the new number of digits to be draw in view before comma.
     */
     public void setIntDigits(int d) {
       intDigits = d;
       decimalFormat = new java.text.DecimalFormat (getFormatString());
     }

     /**
     * Gets the new number of digits to be draw in view before comma.
     */
     public int getIntDigits() {
       return intDigits;
     }

     // unitString  ------------------------------------------------------------
     String unitString = "";

     /**
     * Sets the new unitString.
     */
     public void setUnitString(String s) {
       unitString = s;
     }

     /**
     * Gets the actual unitString.
     */
     public String getUnitString() {
       return unitString;
     }


}
