
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class ProgressView         --------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.vis.output;

import java.awt.*;
import java.util.*;
import java.beans.*;
import javax.swing.*;

/**
* Animation view to visualize a numeric value by a progress bar. 
* Can serve as variable change listeners for numeric variables.
* Paints the numeric value by drawing a bar of appropriate length in the view.  
* <p>
* Use the view by adding it as VariableChangeListener to a numeric variables. 
* <p>
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* <li> set the maximum represented value in the bar by using <code>setMaxNr</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G% 
*/
public class ProgressView extends NumberView {

  /** Default constructor ProgressView */
  public ProgressView() {
      //cRepaintManager.currentManager(this).setDoubleBufferingEnabled(false);
  }

// size  -----------------------------------------------------------------------------------------------------

  /** Gets the minimum size for the LayoutManager */
  public Dimension getMinimumSize() {
    return new Dimension(70,20);
  }


// maxNr -----------------------------------------------------------------------------------------------------

  /**
  * Variable to store a maximum number represented in the progress bar.
  * The bar displayed has its maximum height at this value.
  */
  int maxNr= 10;
  /** Sets the maximum number shown by the bar. */
  public void setMaxNr(int n) { maxNr= n; repaint(); }
  /** Gets the maximum number shown by the bar. */
  public int getMaxNr() { return maxNr; }

    
// PaintValue ------------------------------------------------------------------------------------------------

  /**
  * Paints the numeric value by drawing a bar of appropriate length.
  * @param g the graphics context of the view
  */
  public void paintValue(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    int val;
    if (getNumber() == null)
        val = 0;
    else
        val = getNumber().intValue();
    if (maxNr>0) {
      int currLen= width*val/maxNr;
      if (val>0) {
          g.setColor(getForeground());
          g.fill3DRect(3, 3, currLen, height-17, true);
      }
    }
  }

/*    public static void main(String[] args) {
	final ProgressView p = new ProgressView();
	final ProgressView p1 = new ProgressView();
	final TextView tv = new TextView();
	final HorizontalFlowView hbv = new HorizontalFlowView();
	javax.swing.JFrame f = new javax.swing.JFrame();
	JPanel panel = new JPanel();

	panel.setLayout(new GridLayout(2,1));
	panel.add(p);
	panel.add(hbv);
	p.setMaxNr(100);
	p1.setMaxNr(100);
	tv.setText("test");
	f.getContentPane().add(panel);
	f.setSize(100,100);
	f.setLocation(100,100);
	Thread t = new Thread() {
		public void run() {
		    try {
			while (true) {
			    for (int i=1; i<100; i++) {
				p.setValue(new Integer(i));
				p.repaint(10,10,50,10);
				sleep(1);
			    }
			}
		    } catch (Exception e) {}
		}
};

	Thread t1 = new Thread() {
		public void run() {
		    try {
			sleep(100);
			while (true) {
			    for (int j=1; j<=100; j++) {
 				hbv.setValue(new Double(j));
// 				hbv.variableChange(new sim.kernel.VariableChangeEvent(null,null,new Double(j)));
// 				Rectangle rec = hbv.getUpdateArea();
// 				if (rec != null) {
// 				    System.out.println("repainting with rec  "+rec.x+" "+rec.y+" "+rec.width+" "+rec.height);
// 				    hbv.repaint(rec);
// 				} else {
// 				    System.out.println(j+" --> NULL");
// 				    hbv.repaint();
// 				}
				hbv.repaint();
				sleep(10);
			    }
			}
		    } catch (Exception e) {}
		}
};

	f.show();
	t.start();
 	t1.start();
    }
    
*/
}
